import atexit
import tempfile
import sys
import mock
from selenium import webdriver
import os

def build_mock_mapping(name):
    mock_driver = mock.Mock()
    browser_mapping = {name:mock_driver}
    mock_driver.return_value.name = name
    return browser_mapping

phantom = None
def get_driver():
    global phantom
    if not phantom:
        phantom = webdriver.PhantomJS()
        atexit.register(phantom.quit)
    phantom.delete_all_cookies()
    phantom.switch_to_default_content()
    return phantom


def make_temp_page(src):
    f = tempfile.mktemp(".html")
    fh = open(f, "w")
    fh.write(src.replace("\n",""))
    fh.close()
    atexit.register( lambda: os.remove(f) )
    return "file://%s" % f

def mock_open():
    if sys.version_info >= (3,0,0):
        return mock.patch("builtins.open")
    return mock.patch("__builtin__.open")



import mock
import unittest
from  holmium.core import Page, Element, Elements, Section, Locators
import holmium.core.pageobject
from contextlib import closing

from tests.utils import get_driver, make_temp_page


class BasicSectionIframe(Section):
    element = Element( Locators.CLASS_NAME, "frame_el")
class BasicPageIframe(Page):
    element = Element( Locators.CLASS_NAME, "frame_el" )
    elements = Elements( Locators.CLASS_NAME, "frame_el" )
    frame_1 = BasicSectionIframe(Locators.CLASS_NAME, "section", "frame_1")
    frame_2 = BasicSectionIframe(Locators.CLASS_NAME, "section", "frame_2")
class BasicPage(Page):
    element = Element( Locators.ID, "test_id" )

class IFrameTest(unittest.TestCase):
    def setUp(self):
        self.driver = get_driver()
    def test_basic_po_with_frame(self):
        frame1 = "<html><body><div class='section'><div class='frame_el'>frame 1 el</div></div></body></html>"
        frame2 = "<html><body><div class='section'><div class='frame_el'>frame 2 el</div></div></body></html>"

        uri_frame_1 = make_temp_page(frame1)
        uri_frame_2 = make_temp_page(frame2)

        p1 = '<html><body><iframe id="frame_1" src="%s"/></body></html>' % uri_frame_1
        p2 = '<html><body><iframe id="frame_1" src="%s"></iframe><iframe id="frame_2" src="%s"></iframe></body></html>' % ( uri_frame_1, uri_frame_2)

        driver = get_driver()
        uri = make_temp_page(p1)
        with mock.patch("holmium.core.pageobject.log") as log:
            p = BasicPageIframe(driver, uri, iframe="frame_1")
            self.assertEqual(p.element.text, "frame 1 el")
            self.assertEqual(p.frame_1.element.text, "frame 1 el")
            self.assertTrue(p.frame_2.element == None)
            uri = make_temp_page(p2)
            driver.get(uri)
            self.assertTrue(p.frame_2.element is not None)
            self.assertEqual(p.frame_2.element.text, "frame 2 el")
            self.assertEqual(p.elements[0].text, "frame 1 el")
            self.assertEqual(log.error.call_count, 1)

    def test_mocked_basic_po_with_frame(self):
        with mock.patch('selenium.webdriver.Firefox') as driver:
            with mock.patch('selenium.webdriver.remote.webelement.WebElement') as element:
                element.tag_name = "div"
                element.text = "test_text"
                driver.find_element.return_value = element
                po = BasicPage( driver , iframe='frame')
                self.assertEqual( "test_text",  po.element.text)
                driver.switch_to_frame.assert_called_with("frame")
                self.assertEqual(driver.switch_to_frame.call_count, 1)
                self.assertEqual(driver.switch_to_default_content.call_count, 1)




from holmium.core import Page, Element, Locators, Elements, ElementMap, Section, Sections
from holmium.core.cucumber import init_steps
init_steps()

class TestSection(Section):
    el = Element(Locators.NAME, "el")
    els = Elements(Locators.NAME, "els")
    elmap = ElementMap(Locators.NAME, "elmap")

class TestSections(Sections):
    el = Element(Locators.NAME, "el")


class TestPage(Page):
    el = Element(Locators.NAME, "el")
    els = Elements(Locators.NAME, "els")
    elmap = ElementMap(Locators.NAME, "elmap")
    sections = TestSections(Locators.NAME, "sections")
    section  = TestSection(Locators.NAME, "section")
    def do_stuff(self, a, b):
        return a+b

    def do_stuff_no_args(self):
        return True

    def do_stuff_var_args(self, *args, **kwargs):
        return args, kwargs



"""
utilities for managing the holmium runtime
environment
"""
import atexit
import copy
import sys
from nose.plugins.skip import SkipTest
from six import reraise
from holmium.core.config import configure


class LazyWebDriver(object):
    """
    lazily initializes the webdriver on the first
    attribute access
    """
    # pylint: disable=star-args,protected-access
    def __init__(self, driver_cls, holmium_config):
        self._driver_cls = driver_cls
        self._holmium_config = holmium_config
        self._instance = None

    def __getattribute__(self, item):
        safe_getter = lambda i: object.__getattribute__(self, i)
        safe_setter = lambda i, v: object.__setattr__(self, i, v)
        try:
            return object.__getattribute__(self, item)
        except AttributeError:
            instance = safe_getter("_instance")
            if not instance:
                try:
                    args = configure(safe_getter("_holmium_config"))
                    instance = safe_getter("_driver_cls")(**args)
                except:
                    traceback = sys.exc_info()[2]
                    browser = safe_getter("_holmium_config").browser
                    reraise(
                        SkipTest,
                        SkipTest("unable to initialize driver (name: %s)" % (browser.strip() or "None")),
                        traceback
                    )
                safe_getter("_post_create_callback")()
                safe_setter("_instance", instance)

            return getattr(instance, item)

    def __eq__(self, other):
        return other and self._driver_cls == other._driver_cls


    def _post_create_callback(self):
        """
        register the driver to be shutdown on process exit
        """
        atexit.register(object.__getattribute__(self, "safe_quit"))

    def safe_quit(self):
        """
        quit the driver if the instance was initialized
        """
        try:
            instance = object.__getattribute__(self, "_instance")
            if instance:
                instance.quit()
        except SkipTest:  # pragma: no cover
            # absorb since safe_quit is called by holmium itself
            pass  # pragma: no cover
        finally:
            object.__setattr__(self, "_instance", None)

    def safe_clear(self):
        """
        clear the cookies if the driver was initialized
        """
        try:
            self.delete_all_cookies()
        except SkipTest:  # pragma: no cover
            # absorb since safe_quit is called by holmium itself
            pass  # pragma: no cover


class LazyWebDriverList(list):
    """
    fake-ish list to be used to lazily
    initialize new drivers based on the
    first one created
    """
    def __init__(self, *a, **k):
        # reserve idx == 0
        list.__init__(self, *a, **k)
        self.append(0)

    # pylint: disable=protected-access
    def __getitem__(self, item):
        if item == 0:
            return ENV.get("driver", None)
        try:
            return list.__getitem__(self, item)
        except IndexError:
            # copy the driver[0]
            if ENV.get("driver", None):
                driver = copy.copy(ENV["driver"])
                driver._instance = None
                self.insert(item, driver)
                return driver
            return None # pragma: no cover

    def __iter__(self):
        yield ENV["driver"]
        for item in self[1:]:
            yield item


ENV = {}



import unittest
from holmium.core import Page, Element, Locators, Elements, ElementMap
import mock
from tests.utils import get_driver, make_temp_page


class PageTest(unittest.TestCase):
    class BasicPage(Page):
        element = Element( Locators.ID, "test_id" )
    def test_basic_po(self):
        with mock.patch('selenium.webdriver.Firefox') as driver:
            with mock.patch('selenium.webdriver.remote.webelement.WebElement') as element:
                element.tag_name = "div"
                element.text = "test_text"
                driver.find_element.return_value = element
                po = PageTest.BasicPage( driver , iframe = 'some_frame')
                self.assertEqual( "test_text",  po.element.text)

    def test_basic_po_real(self):
        driver = get_driver()
        uri = make_temp_page("""<body><div id='test_id'>test_text</div></body>""")
        po = PageTest.BasicPage(driver, uri)
        self.assertEqual(po.element.text, "test_text")

    def test_all_features(self):
        driver = get_driver()
        content = """
        <div name='e1'>e1</div>
        <div>
            <div name='e2'>e2 - 1
                <div name='e6'>e6</div>
            </div>
            <div name='e2'>e2 - 2</div>
        </div>
        <div>
            <div name='e3'>e3 - 1</div>
            <div name='e3'>e3 - 2</div>
        </div>
        <div name='e4'>e4</div>
        """
        uri = make_temp_page(content)

        driver.get(uri)
        # this may appear weird, but its to test the usecase of providing a WebElement
        # as a base_element. #fml.
        web_element = driver.find_element_by_name("e2")


        class ExhaustivePage(Page):
            element = Element(Locators.NAME, "e1")
            element_invalid = Element(Locators.NAME, "3e")
            elements = Elements(Locators.NAME, "e2")
            elements_invalid = Elements(Locators.NAME, "3e")
            elementmap = ElementMap(Locators.NAME, "e3")
            elementmap_for_ref = ElementMap(Locators.NAME, "e2")
            elementmap_invalid = ElementMap(Locators.NAME, "3e")
            elementmap_raw = { "e4": Element(Locators.NAME, "e4")
                            , "e2": Elements(Locators.NAME, "e2")
                            , "e3": ElementMap(Locators.NAME, "e3")
                            }
            elements_raw = [ Element(Locators.NAME, "e4")
                            , Elements(Locators.NAME, "e2")
                            , ElementMap(Locators.NAME, "e3")
                            ]
            element_list_raw = [
                Element(Locators.NAME, "e4"),
                Element(Locators.NAME, "e6")
            ]
            element_map_raw = {
                "e4":Element(Locators.NAME, "e4"),
                "e6":Element(Locators.NAME, "e6")
            }
            element_second = Element(Locators.NAME, "e2")
            element_ref = Element( Locators.NAME, "e6", base_element = elements[0] )
            element_map_ref = Element( Locators.NAME, "e6", base_element = elementmap_for_ref["e2 - 1\ne6"] )
            element_ref_direct = Element( Locators.NAME, "e6", base_element = element_second )
            element_ref_webelement = Element( Locators.NAME, "e6", base_element = web_element )
            element_ref_invalid = Element( Locators.NAME, "e6", base_element = 42 )

        page = ExhaustivePage(driver)
        self.assertEqual( page.element.text, "e1" )
        self.assertEqual( [el.text for el in page.elements], ["e2 - 1\ne6", "e2 - 2"] )
        self.assertEqual( [(k, k) for k in page.elementmap], [("e3 - 1","e3 - 1"), ("e3 - 2", "e3 - 2")])
        self.assertEqual( [k.text for k in page.element_list_raw], ["e4", "e6"])
        self.assertEqual( sorted([k.text for k in page.element_map_raw.values()]), sorted(["e4", "e6"]))
        v1, v2 = page.element_map_raw.items()
        self.assertEquals(sorted([v1[0], v1[1].text, v2[0], v2[1].text]), sorted(["e4","e4", "e6", "e6"]))
        self.assertEqual( page.elements_raw[0].text, "e4" )
        self.assertEqual( [k.text for k in page.elements_raw[1]], ["e2 - 1\ne6","e2 - 2"])
        self.assertEqual( [k.text for k in page.elements_raw[2].values()], ["e3 - 1","e3 - 2"])
        self.assertEqual( page.elementmap_raw["e4"].text, "e4" )
        self.assertEqual( [k.text for k in page.elementmap_raw["e2"]], ["e2 - 1\ne6", "e2 - 2"])
        self.assertEqual( [k.text for k in page.elementmap_raw["e3"].values()], ["e3 - 1","e3 - 2"])
        self.assertEqual( page.element_ref.text, "e6" )
        self.assertEqual( page.element_ref_direct.text, "e6" )
        self.assertEqual( page.element_ref_webelement.text, "e6" )
        self.assertEqual( page.element_map_ref.text, "e6")
        self.assertRaises( TypeError, lambda: page.element_ref_invalid)
        self.assertEqual( page.element_invalid , None)
        self.assertEqual( page.elements_invalid, [])
        self.assertEqual(page.elementmap_invalid, {})

    def test_fluent(self):
        class FluentPage(Page):
            thing = Element(Locators.NAME, "thing")
            def get_text(self):
                return self.thing.text
            def click_thing(self):
                self.thing.click()
        page = "<div name='thing'>i am thing</div>"
        driver = get_driver()
        uri = make_temp_page(page)
        page = FluentPage(driver, uri)
        self.assertTrue(page.click_thing().get_text(), "i am thing")




import unittest
import selenium.webdriver
from holmium.core import Page, Element, Elements, Locators, ElementMap

class GoogleMain(Page):
    search_box = Element( Locators.NAME, "q", timeout = 1)
    google_buttons = ElementMap ( Locators.CLASS_NAME, "gbts" , timeout = 1 )
    search_results = Elements( Locators.CSS_SELECTOR, "li.g>div.rc", timeout = 1, value = lambda el : {
        "link":el.find_element_by_css_selector("h3.r>a").get_attribute("href"),
        "title":el.find_element_by_css_selector("h3.r>a").text
        })

    def search ( self, query ):
        self.google_buttons["Search"].click() # self.google_buttons behaves just like a dictionary
        self.search_box.clear() # self.search_box is now evaluated directly to a WebElement
        self.search_box.send_keys(query)
        self.search_box.submit()


class TextSearchTest(unittest.TestCase):
    def setUp(self):
        self.driver = selenium.webdriver.Firefox()
        self.page = GoogleMain(self.driver, "http://www.google.com")

    def test_text_search(self):
        self.assertTrue(len(self.page.search("selenium").search_results) > 0)

    def test_text_search_first_result(self):
        self.page.search("selenium") # execute the page object method search
        self.assertEquals( self.page.search_results[0]["title"], "Selenium - Web Browser Automation")
        self.assertEquals( self.page.search_results[0]["link"], "http://docs.seleniumhq.org/")

    def tearDown(self):
        self.driver.quit()



"""

"""
import unittest
import os

import mock

from holmium.core import TestCase, Page, Element, Elements, ElementMap, Locators
from holmium.core.env import ENV
import holmium.core.testcase
from .utils import build_mock_mapping


def runtc(env, validations, validator = lambda c,s:c(s)):
    try:
        _pre_env = dict(os.environ)
        os.environ.update(env)
        class t(TestCase):

            def setUp(self):
                class P(Page):
                    e = Element(Locators.CLASS_NAME, "el")
                    es = Elements(Locators.CLASS_NAME, "el")
                    em = ElementMap(Locators.CLASS_NAME, "el")
                self.page = P(self.driver)
            def runTest(self):
                for validation in validations:
                    validator(validation, self)

        test = t()
        test.setUpClass()
        test.setUp()
        test.runTest()
        test.tearDown()
        test.tearDownClass()
    finally:
        os.environ = _pre_env

class TestCaseTests(unittest.TestCase):
    def setUp(self):
        ENV.clear()

    def test_assert_element_positive(self):
        with mock.patch.dict('holmium.core.testcase.BROWSER_MAPPING', build_mock_mapping("firefox")):
            driver_cls = holmium.core.testcase.BROWSER_MAPPING["firefox"]
            driver_cls.return_value.find_element.return_value = mock.Mock(
                text="foo", size={"width": 1, "height": 1},
                is_displayed=lambda: True, value_of_css_property=lambda p:"test")
            driver_cls.return_value.find_elements.return_value = [
                mock.Mock(text="foo", is_displayed=lambda:True),
                mock.Mock(text="bar", is_displayed=lambda:True)]
            validations = [
                lambda s: s.assertElementTextEqual(s.page.e, "foo"),
                lambda s: s.assertElementSize(s.page.e, 1,1),
                lambda s: s.assertElementCSS(s.page.e, "background-color", "test"),
                lambda s: s.assertElementDisplayed(s.page.e),
                lambda s: s.assertElementsDisplayed(s.page.es),
                lambda s: s.assertElementsDisplayed(s.page.em)
            ]
            runtc({"HO_BROWSER":"firefox"}, validations)

    def test_assert_element_negative(self):
        with mock.patch.dict('holmium.core.testcase.BROWSER_MAPPING', build_mock_mapping("firefox")):
            driver_cls = holmium.core.testcase.BROWSER_MAPPING["firefox"]
            driver_cls.return_value.find_element.return_value = mock.Mock(
                text="foobar", size={"width": 2, "height": 1},
                is_displayed=lambda: False, value_of_css_property=lambda p:"testing")
            driver_cls.return_value.find_elements.return_value = [
                mock.Mock(text="foo", is_displayed=lambda:False),
                mock.Mock(text="bar", is_displayed=lambda:False)]

            validations = [
                lambda s: s.assertElementTextEqual(s.page.e, "foo"),
                lambda s: s.assertElementSize(s.page.e, height=1,width=1),
                lambda s: s.assertElementCSS(s.page.e, "background-color", "test"),
                #lambda s: s.assertElementDisplayed(s.page.e),
                lambda s: s.assertElementsDisplayed(s.page.es),
                lambda s: s.assertElementsDisplayed(s.page.em)
            ]
            runtc({"HO_BROWSER": "firefox"}, validations,
                  validator=lambda c,s: s.assertRaises(AssertionError, c, s ))



import unittest
import mock
from  holmium.core import Page, Element, Locators, ElementEnhancer, register_enhancer
from selenium.webdriver.support.ui import Select


class ElementEnhancerTest(unittest.TestCase):

    def setUp(self):
        self.driver = mock.Mock()

    def test_builtin_enhancer(self):
        class SimplePage(Page):
            id_el = Element(Locators.ID, "simple_id")

        self.driver.find_element.return_value.tag_name = "select"
        self.driver.find_element.return_value.text = "foo"
        self.driver.find_element.return_value.is_displayed = lambda: True
        p = SimplePage(self.driver)
        self.assertTrue(isinstance(p.id_el, Select))
        self.assertTrue(isinstance(p.id_el, ElementEnhancer))
        self.assertRaises(AttributeError, lambda: p.id_el.jump)
        self.assertEquals(p.id_el.text, "foo")
        self.assertTrue(p.id_el.is_displayed())

    def test_custom_enhancer(self):
        class DivEnhancer(ElementEnhancer):
            __TAG__ = "div"
            def get_class(self):
                return self.element.get_attribute("class")

        register_enhancer(DivEnhancer)

        class SimplePage(Page):
            id_el = Element(Locators.ID, "simple_id")

        self.driver.find_element.return_value.tag_name = "div"
        self.driver.find_element.return_value.text = "foo"
        self.driver.find_element.return_value.get_attribute.return_value = "div-class"

        p = SimplePage(self.driver)

        self.assertEquals(p.id_el.get_class(), "div-class")


    def test_register(self):
        class BadCls:
            pass
        class Almost(ElementEnhancer):
            pass

        self.assertRaises(TypeError, register_enhancer, "test")
        self.assertRaises(TypeError, register_enhancer, BadCls)
        self.assertRaises(AttributeError, register_enhancer, Almost)



"""
implementation of facet bases and builtin facets
"""

import inspect
import weakref

from abc import ABCMeta, abstractmethod
import re

# pylint: disable=no-name-in-module,abstract-class-not-used
from nose.tools import assert_equals, assert_true
from six import add_metaclass
from .logger import log

@add_metaclass(ABCMeta)
class Facet(object):
    """
    base class to implement an attribute of a page
    """
    __ARGS__ = []
    __OPTIONS__ = {}
    __ALLOW_MULTIPLE__ = True

    def __init__(self, required=True, debug=False, **kwargs):
        self.arguments = {}
        self.options = {}
        self.required = required
        self.debug = debug
        self._parent_class = None
        for arg in self.__ARGS__:
            if arg not in kwargs.keys():
                raise AttributeError("%s is a required argument for %s" % (
                arg, self.__class__.__name__))
            else:
                self.arguments[arg] = kwargs[arg]
                kwargs.pop(arg)
        for arg in self.__OPTIONS__:
            if arg in kwargs.keys():
                self.options[arg] = kwargs[arg]
                kwargs.pop(arg)
            else:
                self.options[arg] = self.__OPTIONS__[arg]
        if kwargs:
            raise AttributeError("unknown argument(s) to %s (%s)" % (
            self.__class__.__name__, ",".join(kwargs.keys())))

    def register(self, obj):
        """
        registers a :class:`Facet` on an object

        :param holmium.core.facets.Faceted obj: the object to register the
         facet on.
        """
        if inspect.isclass(obj):
            obj.get_class_facets().append(self)
            self.parent_class = obj
        else:
            obj.get_instance_facets().append(self)
            self.parent_class = obj.__class__


    def __call__(self, obj):
        self.register(obj)
        return obj

    def get_name(self):
        """
        returns the class name of the facet
        """
        return self.__class__.__name__

    @property
    def parent_class(self):
        """
        returns the parent class
        """
        return self._parent_class()

    @parent_class.setter
    def parent_class(self, parent):
        """
        sets the parent class
        """
        self._parent_class = weakref.ref(parent)

    def get_parent_name(self):
        """
        returns the class name of the parent
        """
        return (self.parent_class and self.parent_class.__name__) or None

    @abstractmethod
    def evaluate(self, driver):
        """
        evaluate whether this facet holds true. Raise an Exception
        if not.

        :param selenium.webdriver.remote.webdriver.WebDriver driver: the
         webdriver
        """


class FacetError(Exception):
    """
    exception raised when a facet has an error
    or can't complete

    :param holmium.core.facets.Facet facet: the facet that failed to evaluate
    :param exceptions.Exception exc: the inner exception that caused the failure
    """

    def __init__(self, facet, exc=None):
        self.message = "%s failed to exhibit facet %s" % (
            facet.get_parent_name(), facet.get_name())
        if exc:
            self.message += " with error %s" % exc
        super(FacetError, self).__init__(self.message)


class FacetCollection(list):
    """
    utility collection class for pageobjects to encapsulate
    facets
    """

    def __init__(self, *a):
        super(FacetCollection, self).__init__(*a)

    @property
    def type_map(self):
        """
        view on the list to help with figuring out
        if a facet of the same type already exists
        """
        type_mmap = {}
        for item in self:
            type_mmap.setdefault(type(item), []).append(item)
        return type_mmap

    def append(self, item):
        """
        overridden add method to pop the last
        item if its type does not support multiple
        facets on the same object.
        """
        if (
            type(item) in self.type_map
            and not type(item).__ALLOW_MULTIPLE__
        ):
            self.remove(self.type_map[type(item)].pop())
        if item not in self:
            super(FacetCollection, self).append(item)



    def evaluate_all(self, driver):
        """
        iterate over all registered :class:`Facet` objects and validate them

        :param selenium.webdriver.remote.webdriver.WebDriver driver:
         the webdriver
        """
        for facet in self:
            try:
                facet.evaluate(driver)
            # pylint: disable=broad-except
            except Exception as _:
                if facet.debug:
                    log.warn(FacetError(facet, _))
                elif facet.required:
                    raise FacetError(facet, _)

class CopyOnCreateFacetCollectionMeta(ABCMeta):
    """
    makes a new copy of any :class:`FacetCollection`
    instances upon creating the class. This is to ensure
    that different derived classes of :class:`Page`
    do not clobber the class facets of the base class.
    """

    def __init__(cls, *args):
        super(CopyOnCreateFacetCollectionMeta, cls).__init__(*args)
        visited = {}
        for superclass in cls.__mro__:
            for key, value in vars(superclass).items():
                if isinstance(value, (FacetCollection)):
                    visited.setdefault(key, FacetCollection())
                    for facet in value:
                        visited[key].append(facet)
                    setattr(cls, key, FacetCollection(visited[key]))

@add_metaclass(CopyOnCreateFacetCollectionMeta)
class Faceted(object):
    """
    mixin for objects that want to have facets registered
    on them.
    """

    def __init__(self):
        self.instance_facets = FacetCollection()
        super(Faceted, self).__init__()

    @classmethod
    def get_class_facets(cls):
        """
        returns the facets registered on the class (presumably
        via a decorator)
        """
        if not hasattr(cls, "class_facets"):
            cls.class_facets = FacetCollection()
        return cls.class_facets

    def get_instance_facets(self):
        """
        returns the facets registered on the instance
        """
        return object.__getattribute__(self, "instance_facets")


    def evaluate(self):
        """
        evaluates all registered facets (class & instance)
        """
        from .pageobject import Page

        safe_get = lambda e: object.__getattribute__(self, e)
        driver = Page.get_driver()
        instance_facets = safe_get("get_instance_facets")()
        class_facets = safe_get("get_class_facets")()

        class_facets.evaluate_all(driver)
        instance_facets.evaluate_all(driver)


class Defer(Facet):
    """
    :param holmium.core.Page page: the page object that is expected to be
     deferred to
    :param function action: a callable that takes the page object instance as
     the first argument
    :param dict action_arguments: (optional) dictionary of arguments to pass to
     `action`
    :param bool debug: if True a failure to evaluate will not result in an
     exception, only a log warning
    :param bool required: if False a failure to evaluate will be treated as a
     noop.
    """
    __ARGS__ = ["page", "action"]
    __OPTIONS__ = {"action_arguments": {}}

    def evaluate(self, driver):
        page_cls = self.arguments["page"]
        page = page_cls(driver)
        return self.arguments["action"](page,
                                        **self.options["action_arguments"]
        )


class Title(Facet):
    """
    enforces the title of the current page.

    :param str title: a regular expression to match the title.
    :param bool debug: if True a failure to evaluate will not result in an
     exception, only a log warning
    :param bool required: if False a failure to evaluate will be treated as
     a noop.
    """
    __ARGS__ = ["title"]
    __ALLOW_MULTIPLE__ = False
    def evaluate(self, driver):
        assert_true(re.compile(self.arguments["title"]).match(driver.title),
                    "title did not match %s" % self.arguments["title"]
        )


class Cookie(Facet):
    """
    enforces the existence (and optionally the value) of a cookie.

    :param str name: name of the cookie
    :param dict value: (optional) dict (or callable) to validate the value of
     the cookie.
    :param bool debug: if True a failure to evaluate will not result in an
     exception, only a log warning
    :param bool required: if False a failure to evaluate will be treated as
     a noop.

    """
    __ARGS__ = ["name"]
    __OPTIONS__ = {"value": None}

    def evaluate(self, driver):
        cookie_value = driver.get_cookie(self.arguments["name"])
        if cookie_value and self.options["value"]:
            if callable(self.options["value"]):
                assert self.options["value"](cookie_value)
            else:
                assert_equals(cookie_value, self.options["value"])
        else:
            assert_true(cookie_value is not None,
                        "cookie %s does not exist" % self.arguments["name"]
            )


class Strict(Facet):
    """
    enforces that every element declared in the :class:`Page` or
    :class:`Section` be present.

    :param bool debug: if True a failure to evaluate will not result in an
     exception, only a log warning
    :param bool required: if False a failure to evaluate will be treated as a
     noop.
    """

    def evaluate(self, driver):
        pass # pragma: no cover

    def __call__(self, obj):
        from .pageobject import ElementGetter

        for element in inspect.getmembers(obj):
            if isinstance(element[1], ElementGetter):
                element[1].is_facet = True
                element[1].is_debug_facet = self.debug
        return obj


class ElementFacet(Facet):
    """
    utility trait used when validating an
    :class:`holmium.core.pageobject.ElementGetter` subclass
    """

    def __init__(self, element, element_name, **kwargs):
        self.element_name = element_name
        self.element = element
        super(ElementFacet, self).__init__(required=True, **kwargs)

    def evaluate(self, driver):
        assert_true(self.element.__get__(self.parent_class, self.parent_class),
                     "No such element"
        )

    def get_name(self):
        return self.element_name

# pylint: disable=invalid-name
cookie = Cookie
strict = Strict
defer = Defer
title = Title




import unittest
import json

from holmium.core import Config
from holmium.core.config import HolmiumConfig, configure


class ConfigTests(unittest.TestCase):
    def test_json_config(self):
        json_cfg = """
    {
        "default": {
            "t3": 4,
            "username":"{{holmium.environment}}user"
        },
        "production":
        {
            "t1": 1,
            "t2": 2
        },
        "development":
        {
            "t1": "{{production['t1']}}",
            "t2": 3
        }
    }
    """
        cfg = Config(json.loads(json_cfg))
        self.assertEqual(cfg["t1"], u"1")
        self.assertEqual(cfg["t2"], 3)
        self.assertEqual(cfg["t3"], 4)
        self.assertEqual(cfg["username"], "developmentuser")
        holmium_vars = {"holmium":{"environment":"production"}}
        cfg = Config(json.loads(json_cfg), holmium_vars)
        self.assertEqual(cfg["t1"], 1)
        self.assertEqual(cfg["t2"], 2)
        self.assertEqual(cfg["t3"], 4)
        self.assertEqual(cfg["username"], "productionuser")


    def test_dict_config(self):
        dct_cfg = {"production":
                {"t2":"{{default['t2']}}", "t3":4},
            "development":
                {"t2":u"{{production['t2']}}"},
            "default":
            {"t1":1,"t2":2,"t3":[1,2,3]}}
        cfg = Config(dct_cfg)
        self.assertEqual(cfg["t1"], 1)
        self.assertEqual(cfg["t2"], u"2")
        self.assertEqual(cfg["t3"], [1,2,3])
        holmium_vars = {"holmium":{"environment":"production"}}
        cfg = Config(dct_cfg, holmium_vars)
        self.assertEqual(cfg["t1"], 1)
        self.assertEqual(cfg["t2"], u"2")
        self.assertEqual(cfg["t3"], 4)

    def test_config_update(self):
        cfg = Config({})
        cfg["foo"] = "{{holmium.environment}}"
        cfg["bar"] = 2

        self.assertEqual(cfg["foo"], "development")
        self.assertEqual(cfg["bar"], 2)

    def test_config_defaults_only(self):
        config = {
                "default":{
                    "base_url": "http://{{holmium.environment}}:3000",
                    "registration_url": "{{default.base_url}}/users/sign_up",
                }
            }
        cfg = Config(config)
        self.assertEqual( cfg["registration_url"], u"http://development:3000/users/sign_up")

    def test_config_default_reference(self):
        config = {
                "default":{
                    "base_url": "http://{{holmium.environment}}:3000",
                    "registration_url": "{{base_url}}/users/sign_up",
                    "random_var":"{{env_random}}"
                },
                "production":{
                    "base_url": "http://awesomesite.com",
                    "env_random":"1",
                    "extended_random": "random_{{random_var}}"
                }
            }
        holmium_vars = {"holmium":{"environment":"production"}}
        cfg = Config(config, holmium_vars)
        self.assertEqual( cfg["registration_url"], u"http://awesomesite.com/users/sign_up")
        self.assertEqual( cfg["extended_random"], u"random_1")
        cfg = Config(config)
        self.assertEqual( cfg["registration_url"], u"http://development:3000/users/sign_up")

    def test_holmium_config_object(self):

        cfg = HolmiumConfig(1,2,3,4,5,6,7)
        self.assertEqual(cfg,  {
            "browser":1,
            "remote":2,
            "capabilities":3,
            "user_agent":4,
            "environment":5,
            "ignore_ssl":6,
            "fresh_instance":7,
        })

        self.assertEqual(cfg.browser, 1)
        self.assertEqual(cfg.remote, 2)
        self.assertEqual(cfg.capabilities, 3)
        self.assertEqual(cfg.user_agent, 4)
        self.assertEqual(cfg.environment, 5)
        self.assertEqual(cfg.ignore_ssl, 6)
        self.assertEqual(cfg.fresh_instance, 7)

        cfg.browser = 2
        self.assertEqual(cfg.browser, 2)
        self.assertEqual(cfg["browser"],2)

        nested = {"holmium":{}}
        nested["holmium"]["config"] = cfg
        nested["holmium"]["config"]["user_agent"] = 1
        self.assertEqual(cfg.user_agent , 1)

        cfg["foo"] = "bar"
        self.assertEqual(cfg.foo, "bar")

    def test_holmium_config_unknown_browser(self):
        cfg = HolmiumConfig(
            "awesome",
            "",
            {},
            "",
            "development",
            False,
            False
        )
        self.assertRaises(RuntimeError, configure, cfg)



import unittest
import holmium.core
class DeprecatedPageTest(unittest.TestCase):
    def test_basic_po(self):
        self.assertRaises(SyntaxError, holmium.core.PageObject)
        self.assertRaises(SyntaxError, holmium.core.PageElement)
        self.assertRaises(SyntaxError, holmium.core.PageElements)
        self.assertRaises(SyntaxError, holmium.core.PageElementMap)
        self.assertRaises(SyntaxError, holmium.core.HolmiumTestCase)



"""
utility callables to be passed to holmium.core.Element(s) only_if parameter
"""
from abc import ABCMeta, abstractmethod
import re
from six import add_metaclass


@add_metaclass(ABCMeta)
class BaseCondition(object):
    """
    base class to implement conditions passed
    to the ``only_if`` parameter of :class:`holmium.core.pageobject.ElementGetter`
    subclasses.
    """
    def __call__(self, element):
        return self.evaluate(element)

    @abstractmethod
    def evaluate(self, element):
        """
        abstract method to be implemented by derived classes.
        """
        raise NotImplementedError


# pylint: disable=invalid-name
class VISIBLE(BaseCondition):
    """
    checks if the element is visible
    """
    def evaluate(self, element):
        return element and element.is_displayed()


class INVISIBLE(BaseCondition):
    """
    checks if the element is invisible
    """
    def evaluate(self, element):
        return not element or not element.is_displayed()

class MATCHES_TEXT(BaseCondition):
    """
    checks if the  element's text matches the provided
    regular expression.
    """
    def __init__(self, expr):
        self.expr = expr
    def evaluate(self, element):
        return element and re.compile(self.expr).match(element.text)





"""

"""
from contextlib import closing
from functools import wraps
import sqlite3
from flask import Flask, Blueprint, request, session, redirect, render_template, flash, make_response, url_for, abort, jsonify
import hashlib

bp = Blueprint("test", __name__)

view_vars = {
    "project_name":"Holmium test suite",
}

class DB:
    def __init__(self):
        self._db = None
    def init_app(self, app):
        self.app = app

    @property
    def db(self):
        if not self._db:
            self._db = sqlite3.connect(self.app.config['DATABASE'], check_same_thread=False)
        return self._db

    def create_db(self):
        with self.app.open_resource('schema.sql', mode='r') as f:
            self.db.cursor().executescript(f.read())
        self.db.commit()

    def get_user(self, email):
        return self.db.execute("select email, password from users where email = ?", (email,)).fetchone()

    def create_user(self, email, password):
        values = (email, hashlib.md5(password.encode("utf-8")).hexdigest())
        return self.db.execute("insert into users (email, password) values(?,?)", values)

    def get_entry(self, entry_name):
        return self.db.execute("select value from entries where link = ?", (entry_name,)).fetchone()

    def get_entries(self):
        return dict(self.db.execute("select link, title from entries").fetchall())


def create_app():
    app = Flask(__name__)
    app.register_blueprint(bp)
    app.config["DEBUG"] = True
    app.config["DATABASE"] = ":memory:"
    app.config["SECRET_KEY"] = "wow"
    db.init_app(app)
    return app

db = DB()

def requires_login(fn):
    @wraps(fn)
    def __inner(*a, **kw):
        def check_cookie():
            try:
                user = db.get_user(session.get("current_user"))
                return user and request.cookies["uid"] == hashlib.md5(
                    session.get("current_user").encode("utf-8")).hexdigest() + ":" + user[1]
            except Exception as e:
                return False
        if not ("uid" in request.cookies and check_cookie()):
            if "current_user" in session:
                session.pop("current_user")
            return redirect("/login?return=%s" % request.url_rule)
        return fn(*a, **kw)
    return __inner

def validate_login(email, password):
    flash_and_back = lambda msg:flash(msg, "warning") or redirect(url_for("test.login"))
    if not email:
        return flash_and_back("Email required")
    if not password:
        return flash_and_back("Password required")
    user = db.get_user(email)
    if user:
        if hashlib.md5(password.encode("utf-8")).hexdigest() == user[1]:
            cookie = ("uid", hashlib.md5(email.encode("utf-8")).hexdigest() + ":" + hashlib.md5(password.encode("utf-8")).hexdigest())
            response = make_response(redirect(request.args.get("return", "/")))
            session["current_user"] = email
            response.set_cookie(*cookie)
            return response
        else:
            return flash_and_back("Invalid password")
    else:
        return flash_and_back("Unknown user")

def validate_signup(email, password):
    flash_and_back = lambda msg:flash(msg, "warning") or redirect(url_for("test.signup"))
    if "current_user" in session:
        flash_and_back("You are already signed in")
    if not email:
        return flash_and_back("Email required")
    if not password:
        return flash_and_back("Password required")
    user = db.get_user(email)
    if user:
        return flash_and_back("User already exists")
    else:
        db.create_user(email, password)
        cookie = ("uid", hashlib.md5(email.encode("utf-8")).hexdigest() + ":" + hashlib.md5(password.encode("utf-8")).hexdigest())
        response = make_response(redirect(url_for("test.index")))
        session["current_user"] = email
        response.set_cookie(*cookie)
        return response

@bp.route("/")
@requires_login
def index():
    return render_template("index.html", callout="Here's a few ways it can be good", links=db.get_entries(), **view_vars)

@bp.route("/login", methods=["GET", "POST"])
def login():
    if request.form:
        return  validate_login(request.form["email"], request.form["password"])
    return render_template("login.html", callout="Sign up", **view_vars)

@bp.route("/signup", methods=["GET", "POST"])
def signup():
    if "current_user" in session:
        return redirect(url_for("test.index"))
    if request.form:
        return  validate_signup(request.form["email"], request.form["password"])
    return render_template("signup.html", callout="Testing is good", **view_vars)

@bp.route("/logout")
def logout():
    if "current_user" in session:
        session.pop("current_user")
    resp = redirect(url_for("test.index"))
    resp.set_cookie("uid", "", expires=0)
    return resp

@bp.route("/reference/<link>")
@requires_login
def reference(link):
    entry =  db.get_entry(link)
    if not entry:
        abort(404)
    else:
        return make_response(jsonify({"link":link, "data": entry[0]}))


if __name__ == "__main__":
    app = create_app()
    db.create_db()
    app.run()



"""

"""
import unittest


class NoseInvalidDriverTest(unittest.TestCase):
    def test_driver_single(self):
        self.driver.get("http://never.com")
    def test_driver_double(self):
        self.drivers[0].get("http://never.com")
        self.drivers[1].get("http://never.com")



"""
implementation of cucumber steps for use with fresher/n
"""
from functools import wraps
import inspect
import re
import time

from selenium.webdriver.common.keys import Keys
# pylint: disable=E0611
from nose.tools import assert_equals, assert_true

from fresher import Transform, When, scc, Then, NamedTransform, ftc
from .pageobject import Registry, Section, Sections


def paramfromconfig(function):
    """
    decorator for internal use. maps arguments from ftc.config
    """
    @wraps(function)
    def __inner(*args):
        """
        wrapper
        """
        # pylint: disable=star-args
        def get_and_set(value):
            """
            lol.
            """
            ftc.config["_holmium_cucumber_temp_var"] = value
            return ftc.config["_holmium_cucumber_temp_var"]
        _args = []
        for arg in args:
            _args.append(get_and_set(arg))
        return function(*_args)
    return __inner


def word_to_index(word):
    """
    convert an index in english (1st, first, 20th, twenty first etc..)
    into a number. I'm sure someone out there has a much cleaner implementation.
    """

    ones = {
        'twenty': 2,
        'thirty': 3,
        'fourty': 4,
        'fifty': 5,
        'sixty': 6,
        'seventy': 7,
        'eighty': 8,
        'ninety': 9}
    ones_rev = dict((v, k) for k, v in ones.items())

    anomolies = {1: "first", 2: "second", 3: "third", 4: "fourth", 5: "fifth",
                 6: "sixth", 7: "seventh", 8: "eighth", 9: "ninth", 10: "tenth",
                 11: "eleventh", 12: "twelvth", 13: "thirteenth",
                 14: "fourteenth", 15: "fifteenth",
                 16: "sixteenth", 17: "seventeenth", 18: "eighteenth",
                 19: "nineteenth"}
    anomolies_rev = dict((v, k) for k, v in anomolies.items())
    rest = {}
    for i in range(20, 99):
        if i % 10 == 0:
            rest[i] = ones_rev[i // 10]
        else:
            first = ones_rev[i // 10]
            second = anomolies[i % 10]
            rest["%s %s" % (first, second)] = i
    easy = re.compile(r"(\d+)(st|rd|th)")

    if easy.match(word):
        return int(easy.findall(word)[0][0]) - 1
    elif word in anomolies_rev:
        return anomolies_rev[word] - 1
    elif word in rest:
        return rest[word] - 1
    else:
        return word

def init_steps():
    """
    required to inject holmium steps into the callers' step module.
    """
    frm = inspect.stack()[1]
    mod = inspect.getmodule(frm[0])
    from fresher.stepregistry import StepImpl, TransformImpl, NamedTransformImpl
    fresher_types = (StepImpl, TransformImpl, NamedTransformImpl)
    for key, value in globals().items():
        if any(((value, _type) for _type in fresher_types)):
            setattr(mod, key, value)


@Transform(r"^the page (\w+)$")
def transform_page(name):
    """
    looks up the page in the Registry
    """
    if name not in Registry.pages:
        raise AttributeError(
            "page object %s not found. did you import it?" % name)
    return Registry.pages[name]


@NamedTransform(r"{element}", r"(element (\w+))", r"element (\w+)")
def transform_element(name):
    """
    looks up the element in the page currently registered on the context
    """
    if not hasattr(scc.page, name):
        raise AttributeError(
            "page object %s does not contain an element named %s" % (
                scc.page.__class__.__name__, name
            )
        )
    element = getattr(scc.page, name)
    return element


@NamedTransform(r"{item_in_elements}", r"((.*?)\s+item\s+in\s+(\w+))",
                r"(.*?)\s+item\s+in\s+(\w+)")
def transform_sub_element(key, name):
    """
    looks up a subelement (either index, key or section memeber)
    """
    if not hasattr(scc.page, name):
        raise AttributeError(
            "page object %s does not contain an element named %s" % (
                scc.page.__class__.__name__, name
            )
        )
    sub_expr = re.compile(r"(\w+)(?:\s* item)? (?:of|for) the (.*)$")
    element = getattr(scc.page, name)
    sub = None
    if sub_expr.match(key):
        sub, key = sub_expr.findall(key)[0]
        sub = word_to_index(sub)
    key = word_to_index(key)
    if isinstance(element, dict):
        sub_element = element[key]
    elif isinstance(element, Section) and not isinstance(element, Sections):
        sub_element = getattr(element, key)
    else:
        sub_element = element[int(key)]
    if sub != None:
        if isinstance(sub_element, (list, dict)):
            return sub_element[sub]
        return getattr(sub_element, sub)
    return sub_element


@When(r"^I?\s*access (the page .*?) at url (.*?)$")
@paramfromconfig
def access_page(page, url):
    """
    loads the pageobject with the given url
    into the scenario context
    """
    scc.page = page(ftc.driver, url)

@When(r"^I?\s*access the url (.*?)$")
@paramfromconfig
def access_url(url):
    """
    switch the driver to the provided url
    """
    ftc.driver.get(url)

@When(r"^I?\s*perform (\w+) with (?:arguments)?\s*(.*?)$")
@paramfromconfig
def page_action_with_args(action, args):
    """
    execute a method of the page with arguments
    """
    if not hasattr(scc.page, action):
        raise AttributeError("page object %s does not contain a method %s" % (
            scc.page.__class__.__name__, action)
        )
    arg_list = [k for k in args.split(",") if k.strip()]
    # pylint: disable=star-args
    getattr(scc.page, action)(*arg_list)

@When(r"^I?\s*perform (\w+)\s*$")
@paramfromconfig
def page_action(action):
    """
    execute a method of the page
    """
    if not hasattr(scc.page, action):
        raise AttributeError("page object %s does not contain a method %s" % (
            scc.page.__class__.__name__, action)
        )
    getattr(scc.page, action)()

@Then(r"(?:the)?\s*{element} should be visible\s*$")
@paramfromconfig
def element_visible(element, *_):
    """
    assert element is visible
    """
    assert_true(element is not None)


@Then(r"(?:the)?\s*{item_in_elements} should be visible\s*$")
@paramfromconfig
def item_in_element_visible(element, *_):
    """
    assert sub element is visible
    """
    assert_true(element is not None)


@Then(r"(?:the)?\s*{element} should have (\d+) items")
@paramfromconfig
def element_count(element, _, count):
    """
    assert number of subelements
    """
    assert_equals(int(count), len(element))


@When(r"^I?\s*type (.*?) in {element}\s*$")
@paramfromconfig
def element_type(text, element, *_):
    """
    type inside the element
    """
    element.send_keys(text)

@When(r"^I?\s*type (.*?) in {item_in_elements}\s*$")
@paramfromconfig
def item_in_elements_type(text, element, *_):
    """
    type inside the sub element
    """
    element.send_keys(text)

@When(r"^I?\s*(?:press|pressed)\s*(?:enter|Enter) in (?:.*?){element}\s*$")
@paramfromconfig
def press_enter(element, *_):
    """
    press the enter key in the element
    """
    element.send_keys(Keys.ENTER)


@When(r"^I?\s*(?:press|pressed)\s*(?:enter|Enter) in {item_in_elements}\s*$")
@paramfromconfig
def press_enter_in_item_in_element(element, *_):
    """
    press the enter key in the sub element
    """
    element.send_keys(Keys.ENTER)


@When(r"^I?\s*click the {element}\s*$")
@paramfromconfig
def element_click(element, *_):
    """
    click the element
    """
    element.click()


@When(r"^I?\s*click the {item_in_elements}\s*$")
@paramfromconfig
def item_in_element_click(element, *_):
    """
    click the sub element
    """
    element.click()


@Then(r"^I?\s*should see the title (.*?)\s*$")
@paramfromconfig
def assert_title(title):
    """
    assert the title of the page
    """
    assert_equals(ftc.driver.title, title)

@Then(r"(?:the)?\s*{element} should have (?:the)?\s*text (.*?)\s*$")
@paramfromconfig
def text_in_element(*args):
    """
    assert the text of the element
    """
    # pylint: disable=unbalanced-tuple-unpacking
    element, _, text = args
    assert_equals(element.text, text)

@Then(r"(?:the)?\s*{item_in_elements} should have (?:the)?\s*text (.*?)\s*$")
@paramfromconfig
def text_in_elements_item(*args):
    """
    assert the text of the sub element
    """
    element, text = args[0], args[-1]
    assert_equals(element.text, text)


@When(r"^I?\s*wait for (\d+) (second|seconds)")
@paramfromconfig
def wait(seconds, *_):
    """
    sleep if you must...
    """
    time.sleep(int(seconds))


@When(r"^I?\s*go (back|forward)")
@paramfromconfig
def back_or_forward(direction):
    """
    move the browser back/forward
    """
    if direction.strip() == "back":
        ftc.driver.back()
    else:
        ftc.driver.forward()




"""
configuration objects for holmium
"""
import sys
import os
import inspect

import jinja2
from selenium import webdriver
from selenium.webdriver import FirefoxProfile


class Config(dict):
    """Dictionary like helper class for maintaining test data configurations
    per environment.

:class:`holmium.core.TestCase` and :class:`holmium.core.HolmiumNose` both
look for either a config.json or config.py file in the same directory as the
test file, and will make a ``config`` object available to the test case
instance.

The :class:`holmium.core.Config` object is aware of the environment
(specified with ``--holmium-env`` when using nose or ``HO_ENV`` as an
environment variable) and will return the config variable from that
environment or from the `default` key.

Values in the config file can use :class:`jinja2.Template` templates to access
either values from itself, environment variables or a select magic holmium
variables: ``holmium.environment``, ``holmium.browser``, ``holmium.user_agent``
and ``holmium.remote``.

Example config structure (which uses a magic variable ``holmium.environment``
and an environment variable ``$PATH``).

JSON

.. code-block:: json

    {
            'default': {  'path':"{{PATH}}"
                        , 'login_url': '{{url}}/{{holmium.environment}}/login'}
                        , 'username' : '{{holmium.environment}}user'}
            ,'production': {'url':'http://prod.com'
                            ,'password': 'sekret'}
            ,'development': {'url':'http://dev.com'
                            ,'password': 'password'}
    }

Python

.. code-block:: python

    config = {
        {
            'default': { 'path':"{{PATH}}"
                        , 'login_url': '{{url}}/{{holmium.environment}}/login'}
                        , 'username' : '{{holmium.environment}}user'}
            ,'production': {'url':'http://prod.com'
                            , 'password': 'sekret'}
            ,'development': {'url':'http://dev.com'
                            , 'password': 'password'}
        }
    }

When accessing ``self.config`` within a test, due to the default:

* ``self.config['path']`` will always return the value of the environment
   variable `PATH`,
* ``self.config['password']`` will always return 'sekret'

if ``HO_ENV`` or ``--holmium-env`` are ``production``:

* ``self.config['username']`` will return ``productionuser``
* ``self.config['password']`` will return ``sekret``
* ``self.config['login_url']`` will return ``http://prod.com/production/login``

if ``HO_ENV`` or ``--holmium-env`` are ``development``:

* ``self.config['username']`` will return ``developmentuser``
* ``self.config['password']`` will return ``password``
* ``self.config['login_url']`` will return ``http://dev.com/development/login``

    """

    # pylint: disable=dangerous-default-value
    def __init__(self, dct,
                 environment={"holmium": {"environment": "development"}}):
        self.env = environment
        dict.__init__(self, dct)

    def __getitem__(self, key):
        """
        override to evaluate the values through the template
        """
        def __render(item, context):
            """
            renders the string given the context using the jinja template
            """
            def _check_string_type(_item):
                """
                meh python2/3 stuff.
                """

                if isinstance(_item, str):
                    return True
                elif sys.version_info < (3, 0, 0):
                    if isinstance(_item, eval("unicode")):
                        return True
                return False

            if _check_string_type(item):
                template = jinja2.Template(item)
                rendered = template.render(context)
                if rendered != item:
                    return __render(rendered, context)
                else:
                    return rendered
            else:
                return item

        env_ctx = dict.setdefault(self, self.env["holmium"]["environment"], {})
        default_ctx = dict.setdefault(self, "default", {})
        try:
            item = env_ctx[key]
        except KeyError:
            item = default_ctx[key]

        context = dict(self)
        context.update(os.environ)
        context.update(self.env)
        context.update(default_ctx)
        context.update(env_ctx)
        return __render(item, context)

    def __setitem__(self, key, value):
        """
        override to put the value in the right environment bucket
        """
        sub_dict = dict.setdefault(self, self.env["holmium"]["environment"], {})
        sub_dict[key] = value


BROWSER_MAPPING = {"firefox": webdriver.Firefox,
                   "chrome": webdriver.Chrome,
                   "safari": webdriver.Safari,
                   "ie": webdriver.Ie,
                   "opera": webdriver.Opera,
                   "remote": webdriver.Remote,
                   "phantomjs": webdriver.PhantomJS,
                   "iphone": webdriver.Remote,
                   "ipad": webdriver.Remote,
                   "android": webdriver.Remote}

#:
CAPABILITIES = {"firefox": webdriver.DesiredCapabilities.FIREFOX,
                "chrome": webdriver.DesiredCapabilities.CHROME,
                "safari": webdriver.DesiredCapabilities.SAFARI,
                "ie": webdriver.DesiredCapabilities.INTERNETEXPLORER,
                "opera": webdriver.DesiredCapabilities.OPERA,
                "phantomjs": webdriver.DesiredCapabilities.PHANTOMJS,
                "iphone": webdriver.DesiredCapabilities.IPHONE,
                "ipad": webdriver.DesiredCapabilities.IPAD,
                "android": webdriver.DesiredCapabilities.ANDROID}


class HolmiumConfig(dict):
    """
    utility class for storing holmium configuration options strictly.
    The class behaves like a dictionary after construction
    with the additional behavior that any attributes set on it are available
    as keys in the dictionary and vice versa.
    """
    # pylint: disable=unused-argument,too-many-arguments,star-args
    def __init__(self, browser, remote, capabilities, user_agent, environment,
                 ignore_ssl, fresh_instance):
        data = {}
        for arg in inspect.getargspec(HolmiumConfig.__init__).args[1:]:
            setattr(self, arg, locals()[arg])
            data[arg] = locals()[arg]
        super(HolmiumConfig, self).__init__(**data)


    def __setattr__(self, key, value):
        super(HolmiumConfig, self).__setattr__(key, value)
        super(HolmiumConfig, self).__setitem__(key, value)

    def __setitem__(self, key, value):
        super(HolmiumConfig, self).__setattr__(key, value)
        super(HolmiumConfig, self).__setitem__(key, value)

# pylint: disable=too-few-public-methods
class DriverConfig(object):
    """
    base class for configuring a webdriver
    """

    def __call__(self, config, args):
        return args


class FirefoxConfig(DriverConfig):
    """
    configuration for firefox
    """
    def __call__(self, config, args):
        profile = FirefoxProfile()
        if config.user_agent:
            profile.set_preference("general.useragent.override",
                                   config.user_agent)
        if config.ignore_ssl:
            profile.accept_untrusted_certs = True
        args["firefox_profile"] = profile
        args["capabilities"] = args["desired_capabilities"]
        args.pop("desired_capabilities")
        return args


class ChromeConfig(DriverConfig):
    """
    configuration for chrome
    """
    def __call__(self, config, args):
        args["desired_capabilities"].setdefault("chrome.switches", [])
        if config.user_agent:
            args["desired_capabilities"]["chrome.switches"].append(
                "--user-agent=%s" % config.user_agent)
        if config.ignore_ssl:
            args["desired_capabilities"]["chrome.switches"].append(
                "--ignore-certificate-errors")

        return super(ChromeConfig, self).__call__(config, args)


class PhantomConfig(DriverConfig):
    """
    configuration for phantomjs
    """
    def __call__(self, config, args):
        if config.ignore_ssl:
            args.setdefault("service_args", []).append(
                "--ignore-ssl-errors=true")
        return super(PhantomConfig, self).__call__(config, args)


class RemoteConfig(DriverConfig):
    """
    configuration for remote driver (and anything that doesnt have a
    specific configuration)
    """
    def __call__(self, config, args):
        if config.browser == "firefox":
            if "firefox_profile" in args:
                args["browser_profile"] = args["firefox_profile"]
                args.pop("firefox_profile")
            args["desired_capabilities"] = args["capabilities"]
            args.pop("capabilities")
        args["command_executor"] = config.remote
        return super(RemoteConfig, self).__call__(config, args)


CONFIGURATOR_MAPPER = {
    "firefox": FirefoxConfig(),
    "chrome": ChromeConfig(),
    "phantomjs": PhantomConfig(),
    "remote": RemoteConfig()
}


def configure(config):
    """
    sets up the arguments required by the specific
    :class:`selenium.webdriver.Webdriver` instance
    based on the :class:`holmium.core.config.HolmiumConfig`
    object that is passed in.
    """
    if config.browser not in BROWSER_MAPPING.keys():
        raise RuntimeError("unknown browser %s" % config.browser)
    merged_capabilities = CAPABILITIES[config.browser]
    merged_capabilities.update(config.capabilities)
    args = {"desired_capabilities": merged_capabilities}
    if config.browser in CONFIGURATOR_MAPPER:
        args = CONFIGURATOR_MAPPER[config.browser](config, args)
    if config.remote:
        args = CONFIGURATOR_MAPPER["remote"](config, args)

    return args




import unittest
import hiro
from holmium.core import Elements, Element, Locators, Page
from tests.utils import get_driver, make_temp_page


class ElementsTest(unittest.TestCase):
    def setUp(self):
        self.driver = get_driver()

    def test_basic_element(self):
        class SimplePage(Page):
            el_list_default = Elements(Locators.CLASS_NAME, "simple_class")
            el_list_valuemapper = Elements(Locators.CLASS_NAME, "simple_class"
                                                            , value = lambda el: el.find_element_by_tag_name("a").text)
            el_list_valuemapper_complex = Elements(Locators.CLASS_NAME, "simple_class"
                                                                    , value = lambda el : {
                                                                        "link":el.find_element_by_tag_name("a").get_attribute("href")
                                                                        ,"text" : el.find_element_by_tag_name("a").text } )
            first_el = Element( Locators.TAG_NAME, "a" , base_element = el_list_default[0] )

        uri = make_temp_page("""
            <body>
                <div class="simple_class">
                    simple class el 1
                    <a href="http://el1.com/">element 1</a>
                </div>
                <div class="simple_class">
                    simple class el 2
                    <a href="http://el2.com/">element 2</a>
                </div>
                <div class="simple_class">
                    simple class el 3
                    <a href="http://el3.com/">element 3</a>
                </div>
            </body>
        """)
        page = SimplePage(self.driver, uri)
        self.assertEqual([k.text for k in page.el_list_default],
                          ["simple class el 1 element 1",
                           "simple class el 2 element 2",
                           "simple class el 3 element 3"])
        self.assertEqual(page.el_list_valuemapper,
                          ["element 1", "element 2", "element 3"])
        self.assertEqual(page.el_list_valuemapper_complex,
                          [{"link": "http://el1.com/", "text": "element 1"},
                           {"link": "http://el2.com/", "text": "element 2"},
                           {"link": "http://el3.com/", "text": "element 3"}])
        self.assertEqual(page.first_el.text, "element 1")


    def test_missing_elements(self):
        class SimplePage(Page):
            el_list = Elements(Locators.CLASS_NAME,
                                            "element")
            el_list_wait = Elements(
                Locators.CLASS_NAME, "elements", timeout=1)
            el_list_only_if = Elements(
                Locators.CLASS_NAME, "elements", timeout=1,
                only_if=lambda els: len(els) == 1)

        uri = make_temp_page("""
            <body>
                <div class="_">
                </div>
            </body>
        """)
        with hiro.Timeline().scale(10):
            page = SimplePage(self.driver, uri)
            self.assertEqual(page.el_list, [])
            self.assertEqual(page.el_list_wait, [])
            self.assertEqual(page.el_list_only_if, [])
            uri = make_temp_page("""
                <body>
                    <div class="elements">
                        element_text
                    </div>
                </body>
            """)
            page = SimplePage(self.driver, uri)
            self.assertEqual(page.el_list, [])
            self.assertEqual(len(page.el_list_wait), 1)
            self.assertEqual(len(page.el_list_only_if), 1)



"""
implementation of page objects, element(s) and sections
"""


import inspect
import weakref
import types
import threading
import contextlib
import collections
from functools import wraps

import selenium.webdriver.common.by
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import TimeoutException, NoSuchFrameException
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.remote.webelement import WebElement
from six import add_metaclass
from .enhancers import get_enhancers

if hasattr(collections, "OrderedDict"):
    OrderedDict = collections.OrderedDict # pragma: no cover
else:
    from ordereddict import OrderedDict # pragma: no cover

from .facets import Faceted, ElementFacet, CopyOnCreateFacetCollectionMeta
from .logger import log

# pylint: disable=unnecessary-lambda,too-few-public-methods,too-many-arguments
class Locators(selenium.webdriver.common.by.By):
    """
    proxy class to access locator types
    """
    pass

class ElementList(list):
    """
    proxy to a standard list which would be stored in
    a :class:`holmium.core.Page`.
    """

    def __init__(self, instance, *args, **kwargs):
        self.instance = weakref.ref(instance)
        list.__init__(self, *args, **kwargs)

    def __getitem__(self, index):
        return list.__getitem__(self, index).__get__(self.instance(),
                                                     self.instance().__class__)

    def __iter__(self):
        for idx in range(len(self)):
            yield self[idx]

class ElementDict(dict):
    """
    proxy to a standard dict which would be stored in
    a :class:`holmium.core.Page`.
    """

    def __init__(self, instance, *args, **kwargs):
        self.instance = weakref.ref(instance)
        dict.__init__(self, *args, **kwargs)

    def __getitem__(self, key):
        return dict.__getitem__(self, key).__get__(self.instance(),
                                                   self.instance().__class__)
    def values(self):
        return [self[key] for key in self]

    def items(self):
        return [(key, self[key]) for key in self.keys()]

class Registry(CopyOnCreateFacetCollectionMeta):
    """
    simple meta class to keep track of all page objects registered
    """
    pages = {}
    def __new__(mcs, *args, **kwargs):
        page = super(Registry, mcs).__new__(mcs, *args, **kwargs)
        Registry.pages[args[0]] = page
        return page


@add_metaclass(Registry)
class Page(Faceted):
    """
    Base class for all page objects to extend from.
    void Instance methods implemented by subclasses are provisioned
    with fluent wrappers to facilitate with writing code such as::

        class Google(Page):
            def query(self):
                ....

            def submit(self):
                ....

            def get_results(self):
                ....

        assert len(Google().query("page objects").submit().get_results()) > 0

    """
    local = threading.local()

    def __init__(self, driver, url=None, iframe=None):
        # pylint: disable=too-many-branches
        super(Page, self).__init__()
        self.driver = driver
        self.touched = False
        self.initialized = False
        if url:
            self.home = url
        elif driver.current_url:
            self.home = driver.current_url
        else:
            self.home = None

        self.iframe = iframe
        def update_element(element, name):
            """
            check if the element is a facet
            and register it.
            """
            if issubclass(element.__class__, ElementGetter):
                element.iframe = self.iframe
                if element.is_facet:
                    facet = ElementFacet(element,
                                         name,
                                         debug=element.is_debug_facet
                    )
                    facet.register(self)
                return True
            return False
        for element in inspect.getmembers(self.__class__):
            if issubclass(element[1].__class__, list):
                hit = True
                for item in element[1]:
                    hit &= update_element(item, element[0])
                if hit:
                    self.__setattr__(element[0], ElementList(self, element[1]))
            elif issubclass(element[1].__class__, dict):
                hit = True
                for item in element[1].values():
                    hit &= update_element(item, element[0])
                if hit:
                    self.__setattr__(element[0], ElementDict(self, element[1]))
            else:
                update_element(element[1], element[0])

        if url:
            self.driver.get(url)
        self.initialized = True

    @contextlib.contextmanager
    def scope(self):
        """
        context manager to manage the current webdriver
        in use.
        """
        Page.local.driver = object.__getattribute__(self, "driver")
        yield

    @classmethod
    def get_driver(cls):
        """
        returns the thread local driver
        """
        return cls.local.driver

    def go_home(self):
        """
        returns the page object to the url it was initialized with
        """
        self.driver.get(self.home)

    def __getattribute__(self, key):
        """
        to enable fluent access to page objects, instance methods that
        don't return a value, instead return the page object instance.
        """

        attr_getter = lambda key: object.__getattribute__(self, key)
        attr_setter = lambda key, value: object.__setattr__(self, key, value)

        with attr_getter("scope")():
            if not attr_getter("touched") and attr_getter("initialized"):
                attr_getter("evaluate")()
                attr_setter("touched", True)
            attr = attr_getter(key)
            # check if home url is set, else update.
            if not attr_getter("home"):
                log.debug(
                    "home url not set, attempting to update.")
                attr_setter("home", attr_getter("driver").current_url)

            if isinstance(attr, types.MethodType):
                @wraps(attr)
                def wrap(*args, **kwargs):
                    """
                    fluent wrapper
                    """
                    resp = attr(*args, **kwargs)
                    if None == resp:
                        resp = self
                    return resp

                return wrap
            return attr



class ElementGetter(object):
    """
    internal class to encapsulate the logic used by
    :class:`holmium.core.Element`
    & :class:`holmium.core.Elements`
    """

    # pylint: disable=too-many-instance-attributes
    def __init__(self, locator_type,
                 query_string,
                 base_element=None,
                 timeout=0,
                 value=lambda el: el,
                 only_if=lambda el: el is not None,
                 facet=False):
        """
        :param holmium.core.Locators locator_type: selenium locator to use when
         locating the element
        :param str query_string: the value to pass to the locator
        :param holmium.core.Element base_element: a reference to another element
         under which to locate this element.
        :param int timeout: time to implicitely wait for the element
        :param lambda value: transform function for the value of the element.
         The located :class:`selenium.webdriver.remote.webelement.WebElement`
         instance is passed as the only argument to the function.
        :param function only_if: extra validation function that is called
         repeatedly until :attr:`timeout` has elapsed. If not provided the
         default function used checks that the element is present.
         The located :class:`selenium.webdriver.remote.webelement.WebElement`
         instance is passed as the only argument to the function.
        :param bool facet: flag to  treat this element as a facet.
        """
        self.query_string = query_string
        self.locator_type = locator_type
        self.timeout = timeout
        self.driver = None
        self.iframe = None
        self.base_element = base_element
        self.value_mapper = value
        self.root_fn = lambda: Page.get_driver()
        self.only_if = only_if
        log.debug("locator:%s, query_string:%s, timeout:%d" %
                               (locator_type, query_string, timeout))
        self.is_facet = facet
        self.is_debug_facet = False

    @property
    def root(self):
        "returns the root webelement"
        return self.root_fn()

    @root.setter
    def root(self, root_fn):
        "sets the root element extraction function"
        self.root_fn = root_fn

    def _get_element(self, method=None):
        """
        extracts the webelement(s)
        :param function method: the method used to query the webdriver
        """
        if self.base_element:
            if isinstance(self.base_element, types.LambdaType):
                _ = self.base_element()
                _meth = getattr(_, method.__name__)
            elif isinstance(self.base_element, Element):
                _meth = getattr(self.base_element.__get__(self, self.__class__),
                                method.__name__)
            elif isinstance(self.base_element, WebElement):
                _meth = getattr(self.base_element, "find_element")
            else:
                raise TypeError("invalid base_element type (%s) used" % (
                    type(self.base_element)))
        else:
            _meth = method
        log.debug(
            "looking up locator:%s, query_string:%s, timeout:%d" %
            (self.locator_type, self.query_string, self.timeout))

        if self.iframe:
            Page.local.driver.switch_to_default_content()
            Page.local.driver.switch_to_frame(self.iframe)

        if self.timeout:
            try:
                def callback(_):
                    """
                    timeout & only_if explicit wait.
                    """
                    return _meth(self.locator_type, self.query_string) and \
                        self.only_if(
                            _meth(self.locator_type, self.query_string)
                        )

                WebDriverWait(self.root, self.timeout).until(callback)
            except TimeoutException:
                log.debug(
                    "unable to find element %s after waiting for %d seconds" % (
                        self.query_string, self.timeout)
                )
                raise
        return _meth(self.locator_type, self.query_string)
    @classmethod
    def enhance(cls, element):
        """
        incase a higher level abstraction for a WebElement is available
        we will use that in Pages. (e.g. a select element is converted into
        :class:`selenium.webdriver.support.ui.Select`)
        """
        for enhancer in get_enhancers():
            if enhancer.matches(element):
                return enhancer(element)
        return element

class Element(ElementGetter):
    """
    Utility to get a :class:`selenium.webdriver.remote.webelement.WebElement`
    by querying via one of :class:`holmium.core.Locators`

    :param holmium.core.Locators locator_type: selenium locator to use when
     locating the element
    :param str query_string: the value to pass to the locator
    :param holmium.core.Element base_element: a reference to another element
     under which to locate this element.
    :param int timeout: time to implicitely wait for the element
    :param lambda value: transform function for the value of the element.
     The located :class:`selenium.webdriver.remote.webelement.WebElement`
     instance is passed as the only argument to the function.
    :param function only_if: extra validation function that is called repeatedly
     until :attr:`timeout` elapases. If not provided the default function used
     checks that the element is present. The located
     :class:`selenium.webdriver.remote.webelement.WebElement`
     instance is passed as the only argument to the function.
    :param bool facet: flag to  treat this element as a facet.
    """

    def __get__(self, instance, owner):
        if not instance:
            return self
        try:
            return self.value_mapper(
                self.enhance(self._get_element(self.root.find_element))
            ) if self.root else None
        except (NoSuchElementException, TimeoutException):
            return None


class Elements(ElementGetter):
    """
    Utility to get a collection of
    :class:`selenium.webdriver.remote.webelement.WebElement` objects by
    querying via one of :class:`holmium.core.Locators`

    :param holmium.core.Locators locator_type: selenium locator to use when
     locating the element
    :param str query_string: the value to pass to the locator
    :param holmium.core.Element base_element: a reference to another element
     under which to locate this element.
    :param int timeout: time to implicitely wait for the element
    :param lambda value: transform function for each element in the collection.
     The located :class:`selenium.webdriver.remote.webelement.WebElement`
     instance is passed as the only argument to the function.
    :param function only_if: extra validation function that is called repeatedly
     until :attr:`timeout` elapses. If not provided the default function used
     checks that all the elements are present. The list of located
     :class:`selenium.webdriver.remote.webelement.WebElement`
     instances is passed as the only argument to the function.
    :param bool facet: flag to  treat this element as a facet.
    """
    # pylint: disable=incomplete-protocol,line-too-long
    def __init__(self, locator_type,
                 query_string=None,
                 base_element=None,
                 timeout=0,
                 value=lambda el: el,
                 only_if=lambda el: el is not None,
                 facet=False):
        super(Elements, self).__init__(locator_type, query_string,
                                       base_element=base_element,
                                       timeout=timeout,
                                       facet=facet, value=value,
                                       only_if=only_if)

    def __getitem__(self, idx):
        return lambda: self.__get__(self, self.__class__)[idx]

    def __get__(self, instance, owner):
        if not instance:
            return self
        try:
            return [self.value_mapper(self.enhance(el)) for el in
                self._get_element(self.root.find_elements)] if self.root else []
        except (NoSuchElementException, TimeoutException):
            return []

class ElementMap(Elements):
    """
    Used to create dynamic dictionaries based on an element locator specified by
    one of :class:`holmium.core.Locators`.

    The wrapped dictionary is an :class:`collections.OrderedDict` instance.

    :param holmium.core.Locators locator_type: selenium locator to use when
     locating the element
    :param str query_string: the value to pass to the locator
    :param holmium.core.Element base_element: a reference to another element
     under which to locate this element.
    :param int timeout: time to implicitely wait for the element
    :param bool facet: flag to  treat this element as a facet.
    :param lambda key: transform function for mapping a key to a WebElement in
     the collection. The located
     :class:`selenium.webdriver.remote.webelement.WebElement` instance is passed
     as the only argument to the function.
    :param lambda value: transform function for the value when accessed via the
     key. The located :class:`selenium.webdriver.remote.webelement.WebElement`
     instance is passed as the only argument to the function.
    :param function only_if: extra validation function that is called repeatedly
     until :attr:`timeout`. If not provided the default function used checks
     that all the elements are present. The list of located
     :class:`selenium.webdriver.remote.webelement.WebElement` instances is
     passed as the only argument to the function.
    """
    # pylint: disable=incomplete-protocol,line-too-long
    def __init__(self, locator_type,
                 query_string=None,
                 base_element=None,
                 timeout=0,
                 key=lambda el: el.text,
                 value=lambda el: el,
                 only_if=lambda el: el is not None,
                 facet=False):
        super(ElementMap, self).__init__(locator_type, query_string,
                                         base_element,
                                         timeout, facet, only_if=only_if)
        self.key_mapper = key
        self.value_mapper = value

    def __get__(self, instance, owner):
        if not instance:
            return self
        try:
            return OrderedDict(
            (self.key_mapper(el), self.value_mapper(self.enhance(el))) for el in
            self._get_element(self.root.find_elements)) if self.root else {}
        except (NoSuchElementException, TimeoutException):
            return {}

    def __getitem__(self, key):
        return lambda: self.__get__(self, self.__class__)[key]


class Section(Faceted):
    """
    Base class to encapsulate reusable page sections::

        class MySection(Section):
            things = Elements( .... )

        class MyPage(Page):
            section_1 =  MySection(Locators.CLASS_NAME, "section")
            section_2 =  MySection(Locators.ID, "unique_section")

    """

    def __init__(self, locator_type, query_string, iframe=None, timeout=0):

        super(Section, self).__init__()
        self.touched = False
        self.locator_type = locator_type
        self.query_string = query_string
        self.iframe = iframe
        self.timeout = timeout
        self.__root_val = None
        self.element_members = {}
        for element in inspect.getmembers(self.__class__):
            if issubclass(element[1].__class__, ElementGetter):
                self.element_members[element[0]] = element[1]
                if element[1].is_facet:
                    facet = ElementFacet(element[1],
                                         element[0],
                                         debug=element[1].is_debug_facet
                    )
                    facet.register(self)

    def __get__(self, instance, owner):
        for element in self.element_members.values():
            element.root = lambda: self.root
        return self

    def __getattribute__(self, item):
        attr_getter = lambda key: super(Section, self).__getattribute__(key)
        attr_setter = lambda key, value: super(Section, self).__setattr__(key,
                                                                          value)
        members = attr_getter("element_members")
        touched = attr_getter("touched")
        if not touched and item in members:
            attr_getter("evaluate")()
            attr_setter("touched", True)
        return attr_getter(item)


    @property
    def root(self):
        """
        returns the element the section is rooted at
        """
        if self.iframe:
            try:
                Page.get_driver().switch_to_default_content()
                Page.get_driver().switch_to_frame(self.iframe)
            except NoSuchFrameException:
                log.error(
                    "unable to switch to iframe %s" % self.iframe)
        try:
            if not self.__root_val:
                WebDriverWait(Page.get_driver(), self.timeout).until(
                    lambda _: Page.get_driver().find_element(self.locator_type,
                                                          self.query_string)
                )
            return self.__root_val or Page.get_driver().find_element(
                self.locator_type, self.query_string
            )
        except (NoSuchElementException, TimeoutException):
            return None

    @root.setter
    def root(self, val):
        """
        sets the root of the section
        """
        self.__root_val = val


class Sections(Section, collections.Sequence):
    """
    Base class for an Iterable view of a collection of
    :class:`holmium.core.Section` objects.
    """

    def __init__(self, locator_type, query_string, iframe=None, timeout=0):
        super(Sections, self).__init__(locator_type, query_string, iframe, timeout)

    def __getelements__(self):
        if self.timeout:
            try:
                WebDriverWait(Page.get_driver(), self.timeout).until(
                    lambda _: Page.get_driver().find_elements(self.locator_type,
                                                              self.query_string)
                )
            except TimeoutException:
                log.debug(
                    "unable to find element %s after waiting for %d seconds" % (
                        self.query_string, self.timeout)
                )
        return Page.get_driver().find_elements(self.locator_type,
                                               self.query_string)

    def __iter__(self):
        for element in self.__getelements__():
            self.root = element
            yield self

    def __len__(self):
        return len(self.__getelements__())

    def __getitem__(self, idx):
        _idx = 0
        for _ in self:
            if idx == _idx:
                break
            _idx += 1
            if idx > _idx:
                raise IndexError("Sections index (%d) out of range" % idx)
        if len(self) == 0:
            raise IndexError("Sections index (%d) out of range" % idx)
        return self



"""
holmium.core

"""
from .config import Config
from .logger import log
from .testcase import TestCase
from .pageobject import Locators, Page, Section, Sections, Element, Elements
from .pageobject import ElementMap
from .noseplugin import HolmiumNose
from .deprecated import PageObject, PageElement, PageElements
from .deprecated import PageElementMap, HolmiumTestCase
from .enhancers import register_enhancer, get_enhancers
from .enhancers import reset_enhancers, ElementEnhancer


from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



"""

"""
import unittest
import os

import mock
from nose.plugins import PluginTester
from fresher.noseplugin import FresherNosePlugin

import holmium.core
import holmium.core.noseplugin
from tests import utils


support = os.path.join(os.path.dirname(__file__), "support")

class TestFresherIntegration(PluginTester, unittest.TestCase):
    activate = "--with-holmium"
    args = ['--holmium-environment=development'
            , '--holmium-browser=firefox'
            , '--with-fresher'
    ]
    suitepath = os.path.join(support, 'cucumber')
    plugins = [holmium.core.HolmiumNose(), FresherNosePlugin()]
    def setUp(self):
        self.old_mapping = holmium.core.config.BROWSER_MAPPING
        holmium.core.noseplugin.BROWSER_MAPPING = utils.build_mock_mapping("firefox")
        self.driver = holmium.core.noseplugin.BROWSER_MAPPING["firefox"]
        self.driver.return_value.title = "test"
        e1 = mock.Mock()
        e1.text = "e"
        self.driver.return_value.find_elements.return_value = [e1] * 100
        e1.find_element.return_value.text = "e"
        self.driver.return_value.find_element.return_value.find_element.return_value.text = "e"
        self.driver.return_value.find_element.return_value.text = "e"
        mock_section_element = mock.Mock()
        mock_section_element.text = "e"
        self.driver.return_value.find_element.return_value.find_elements.return_value = [mock_section_element]
        self.sleep_patcher = mock.patch("time.sleep")
        self.sleep_mock = self.sleep_patcher.start()
        super(TestFresherIntegration,self).setUp()
    def runTest(self):
        assert "Ran 10 tests" in self.output, self.output
        assert "FAILED (errors=8)" in self.output, self.output
        assert "'moo'" in self.output, self.output
        assert "TestPage does not contain an element named not_existent" in self.output
        assert "page object FooPage not found. did you import it?" in self.output
        assert "'TestSection' object has no attribute 'moo'" in self.output
        assert "page object TestPage does not contain an element named missing_section" in self.output
        assert str(self.output).count("page object TestPage does not contain a method noop") == 2
        assert "do_stuff() takes" in self.output, self.output
        assert self.driver.call_count == 1, self.driver.call_count
        self.driver.return_value.get.assert_any_call("http://www.google.com")
        self.driver.return_value.get.assert_any_call("http://www.google.com/login")
        assert self.driver.return_value.delete_all_cookies.call_count == 10, self.driver.return_value.delete_all_cookies.call_count
    def tearDown(self):
        holmium.core.noseplugin.BROWSER_MAPPING = self.old_mapping
        self.sleep_patcher.stop()




import unittest
import mock
from  holmium.core import Page, Element, Locators
from selenium.webdriver.support.ui import Select


class ElementWithMockDriverTest(unittest.TestCase):

    def setUp(self):
        self.driver = mock.Mock()

    def test_enhanced_elements(self):
        class SimplePage(Page):
            id_el = Element(Locators.ID, "simple_id")
        self.driver.find_element.return_value.tag_name = "select"
        self.assertTrue(isinstance(SimplePage(self.driver).id_el, Select))


    def test_set_home_after_init(self):
        self.driver.current_url = None
        class SimplePage(Page):
            id_el = Element(Locators.ID, "simple_id")

        page = SimplePage(self.driver)
        self.assertEqual(page.home, None)
        self.driver.current_url = "http://www.google.com"
        self.driver.find_element.return_value.text = "simple"
        # when accessing an element, the home parameter should be updated
        self.assertEqual(page.id_el.text, "simple")
        self.assertEqual(page.home, "http://www.google.com")
        self.assertEqual(self.driver.get.call_count, 0)
        page.go_home()
        self.assertTrue(self.driver.get.call_count, 1)







"""
nose plugin for holmium (or the other way around? :P)
"""
from contextlib import closing
import os
import sys
import json
from nose.plugins.base import Plugin
from nose.plugins.skip import SkipTest
from .config import HolmiumConfig, Config, BROWSER_MAPPING
from .env import ENV, LazyWebDriverList
from holmium.core.env import LazyWebDriver
from .logger import log

# pylint: disable=invalid-name
try:
    from fresher import ftc
except ImportError: # pragma: no cover
    ftc = None # pragma: no cover

def load_source(name, path):
    """
    abstracted out for 2.7 versus 3.x support
    """
    if sys.version_info >= (3, 0, 0):
        # pylint: disable=no-name-in-module
        from importlib import machinery  # pragma: no cover

        loader = machinery.SourceFileLoader(name, path) # pragma: no cover
        return loader.load_module(name)  # pragma: no cover
    else:
        import imp  # pragma: no cover

        return imp.load_source(name, path)  # pragma: no cover

class HolmiumNose(Plugin):
    """
    nose plugin to allow bootstrapping testcases with a selenium driver
    """
    name = "holmium"
    enabled = False

    def __init__(self):
        Plugin.__init__(self)
        self.config = {}
        self.environment = None
        self.logger = log

    def options(self, parser, env):
        """
        Register command line options
        """
        parser.add_option("", "--with-holmium", dest="ho_enabled",
                          action="store_true",
                          help="use holmium to setup test case environment")
        parser.add_option("", "--holmium-environment", dest="ho_env",
                          help="environment to pass to holmium test case"
                               "configuration")
        parser.add_option("", "--holmium-browser", dest="ho_browser",
                          type="choice",
                          choices=list(
                              BROWSER_MAPPING.keys()),
                          help="the selenium driver to invoke")
        parser.add_option("", "--holmium-remote", dest="ho_remote",
                          help="full url to remote selenium instance")
        parser.add_option("", "--holmium-capabilities", dest="ho_cap",
                          help="json dictionary of extra capabilities")
        parser.add_option("", "--holmium-useragent", dest="ho_ua",
                          help="User-agent string to use. Only available"
                               "for firefox & chrome")
        parser.add_option("", "--holmium-ignore-ssl-errors",
                          action="store_true", dest="ho_ignore_ssl",
                          help="Ignore ssl errors while running tests")
        parser.add_option("", "--holmium-browser-per-test",
                          action="store_true", dest="ho_fresh_instance",
                          help="Create a fresh browser per test class",
                          default=False)

    def configure(self, options, conf):
        if options.ho_enabled:
            browser = options.ho_browser or os.environ.get("HO_BROWSER", "")
            environment = options.ho_env or os.environ.get("HO_ENVIRONMENT", "")
            remote_url = options.ho_remote or os.environ.get("HO_REMOTE", "")
            user_agent = options.ho_ua or os.environ.get("HO_USERAGENT", "")
            fresh_instance = options.ho_fresh_instance or bool(
                int(os.environ.get("HO_BROWSER_PER_TEST", 0)))
            ignore_ssl = options.ho_ignore_ssl or os.environ.get(
                "HO_IGNORE_SSL_ERRORS", False)
            if options.ho_cap and os.path.isfile(options.ho_cap):
                caps = json.loads(open(options.ho_cap).read())
            else:
                caps = options.ho_cap and json.loads(options.ho_cap) or {}
            # pylint: disable=attribute-defined-outside-init
            self.holmium_config = holmium_config = HolmiumConfig(browser,
                                                                 remote_url,
                                                                 caps,
                                                                 user_agent,
                                                                 environment,
                                                                 ignore_ssl,
                                                                 fresh_instance)
            # pylint:disable=no-member,attribute-defined-outside-init
            if holmium_config.remote:
                driver_cls = BROWSER_MAPPING["remote"]
            else:
                driver_cls = BROWSER_MAPPING.get(holmium_config.browser, None)
            self.driver = LazyWebDriver(driver_cls, holmium_config)
            self.enabled = True

    # pylint: disable=invalid-name
    def beforeTest(self, test):
        """
        setup the initialization for the driver, and load the config
        """
        if not ("driver" in ENV and ENV["driver"] == self.driver):
            ENV["driver"] = self.driver
        if not "drivers" in ENV:
            ENV["drivers"] = LazyWebDriverList()
        if not hasattr(test.test, "feature"):
            base_file = test.address()[0]
        else:
            base_file = test.test.feature.src_file
        config_path = os.path.join(os.path.split(base_file)[0], "config")
        try:
            config = None
            if os.path.isfile(config_path + ".json"):
                with closing(open(config_path + ".json")) as config_file:
                    config = json.loads(config_file.read())
            elif os.path.isfile(config_path + ".py"):
                if "holmium_testcase_config" in sys.modules:
                    del sys.modules["holmium_testcase_config"]
                config = load_source("holmium_testcase_config",
                                     config_path + ".py").config
            if config:
                self.config = Config(config, {
                "holmium": self.holmium_config})
        except Exception as exc:
            self.logger.debug("unable to load %s" % config_path)
            raise SkipTest(
                "error in loading config file at path %s" % config_path, exc)
        if HolmiumNose.is_freshen_test(test) and ftc:
            ftc.config = self.config
        else:
            setattr(test.test.__class__, "config", self.config)

    # pylint:disable=invalid-name,no-self-use
    def startTest(self, test):
        """
        before each test set the self.driver/self.drivers attribute
        for the test.
        """
        if ENV.get("driver", None):
            if HolmiumNose.is_freshen_test(test) and ftc:
                ftc.driver = ENV["driver"]
                ftc.drivers = ENV["drivers"]
            else:
                setattr(test.test.__class__, "driver", ENV["driver"])
                setattr(test.test.__class__, "drivers", ENV["drivers"])

    def afterTest(self, _):
        """
        run after the test completes. kill the browser or just clear
        cookies depending on config
        """
        # pylint:disable=no-member
        if ENV.get("driver", None) and self.holmium_config.fresh_instance:
            for driver in ENV["drivers"]:
                driver.safe_quit()
        elif ENV.get("driver"):
            for driver in ENV["drivers"]:
                driver.safe_clear()

    @staticmethod
    def is_freshen_test(test):
        """
        checks for tell tale signs of being a fresher test
        """
        return test.address()[1] == "fresher.noseplugin"



import unittest
from holmium.core import Locators, Page, Elements, Section, Sections
import mock
from tests.utils import get_driver, make_temp_page


class BasicSection(Section):
    tokens = Elements(Locators.CSS_SELECTOR,
                                   "div.token")


class BasicSectionList(Sections):
    tokens = Elements(Locators.CSS_SELECTOR,
                                   "div.token")


class BasicPage(Page):
    section = BasicSection(Locators.ID, "section")
    section_2 = BasicSection(Locators.ID, "section_2")
    tokens = Elements(Locators.CLASS_NAME, "token")


class BasicPageWithSections(Page):
    sections = BasicSectionList( Locators.CLASS_NAME, "section", timeout=1)
    missing_sections = BasicSectionList( Locators.CLASS_NAME, "missing_section", timeout=1)

class SectionTest(unittest.TestCase):

    def test_basic_po_real(self):
        driver = get_driver()
        page = """
        <body>
            <div id='section'>
                <div class='token'>
                    <div class='el'>
                        section element 1
                    </div>
                </div>
                <div class='token'>
                    <div class='el'>
                        section element 2
                    </div>
                </div>
            </div>
            <div id='section_2'>
                <div class='token'>
                    <div class='el'>
                        section element 3
                    </div>
                </div>
                <div class='token'>
                    <div class='el'>
                        section element 4
                    </div>
                </div>
            </div>
            <span class='token'>
                <div class='el'>
                    element 5
                </div>
            </span>
        </body>
        """
        uri = make_temp_page(page.strip())
        po = BasicPage(driver, uri)
        self.assertEqual(len(po.tokens), 5)
        self.assertEqual(len(po.section.tokens), 2)
        self.assertEqual(len(po.section_2.tokens), 2)
        for i in range(0, 2):
            self.assertEqual(po.section.tokens[i].text,
                              "section element %s" % (i + 1))
        for i in range(0, 2):
            self.assertEqual(po.section_2.tokens[i].text,
                              "section element %s" % (i + 3))
        self.assertEqual(po.tokens[0].text, 'section element 1')
        self.assertEqual(po.tokens[1].text, 'section element 2')
        self.assertEqual(po.tokens[2].text, 'section element 3')
        self.assertEqual(po.tokens[3].text, 'section element 4')

    def test_basic_po_with_sections(self):
        driver = get_driver()
        page = """
        <body>
            <div class='section'>
                <div class='token'>
                    <div class='el'>
                        section element 1
                    </div>
                </div>
                <div class='token'>
                    <div class='el'>
                        section element 2
                    </div>
                </div>
            </div>
            <div class='section'>
                <div class='token'>
                    <div class='el'>
                        section element 3
                    </div>
                </div>
                <div class='token'>
                    <div class='el'>
                        section element 4
                    </div>
                </div>
            </div>
        </body>
        """
        uri = make_temp_page(page.strip())
        po = BasicPageWithSections(driver, uri)
        counter = 1
        for section in po.sections:
            for token in section.tokens:
                self.assertEqual(token.text, "section element %s" % counter)
                counter += 1
        self.assertEqual(len(po.sections), 2)
        self.assertRaises(IndexError, lambda: po.sections[2])
        self.assertRaises(IndexError, lambda: po.missing_sections[0])

    def test_sections_list_behavior(self):
        with mock.patch('selenium.webdriver.Firefox') as driver:
            element1, element2 = mock.Mock(), mock.Mock()
            element1.tag_name = element2.tag_name = "div"
            element1.text = "element 1"
            element2.text = "element 2"
            element3, element4 = mock.Mock(), mock.Mock()
            element3.tag_name = element4.tag_name = "div"
            element3.text = "element 3"
            element4.text = "element 4"
            driver.find_elements.return_value = [element1, element2]
            element1.find_elements.return_value = [element3, element4]
            element2.find_elements.return_value = [element4, element3]
            po = BasicPageWithSections(driver)
            self.assertEqual("element 3", po.sections[0].tokens[0].text)
            self.assertEqual("element 4", po.sections[1].tokens[0].text)
            self.assertEqual("element 4", po.sections[0].tokens[1].text)
            self.assertEqual("element 3", po.sections[1].tokens[1].text)




import unittest


class NoseConfigTest(unittest.TestCase):
    def test_config_available(self):
        self.assertTrue(self.config != None)
    def test_config_env_value(self):
        self.assertEqual(self.config["var"], 1)
    def test_config_default_value(self):
        self.assertEqual(self.config["foo"], 2)



import unittest
import os
import json
import mock

from nose.plugins import PluginTester
from nose.plugins.skip import Skip

import holmium.core
from holmium.core.env import ENV
import holmium.core.noseplugin

from .utils import build_mock_mapping

support = os.path.join(os.path.dirname(__file__), "support")

class TestOptionsWithFirefox(PluginTester, unittest.TestCase):
    activate = "--with-holmium"
    args = ['--holmium-environment=staging'
            , '--holmium-browser=firefox'
            , '--holmium-useragent="test useragent"'
            , '--holmium-remote=http://nowhere.org'
            , "--holmium-capabilities=%s" % json.dumps({"foo":1})
            , "--holmium-ignore-ssl-errors"
            , "--holmium-browser-per-test"
            ]
    suitepath = os.path.join(support, 'options')
    plugins = [holmium.core.HolmiumNose()]
    def setUp(self):
        self.old_mapping = holmium.core.noseplugin.BROWSER_MAPPING
        holmium.core.noseplugin.BROWSER_MAPPING = build_mock_mapping("remote")
        holmium.core.noseplugin.BROWSER_MAPPING.update(build_mock_mapping("firefox"))
        ENV.clear()
        super(TestOptionsWithFirefox,self).setUp()
    def runTest(self):
        assert "ERROR" not in self.output, self.output
        assert "FAIL" not in self.output, self.output
    def tearDown(self):
        holmium.core.BROWSER_MAPPING = self.old_mapping

class TestOptionsWithChrome(PluginTester, unittest.TestCase):
    activate = "--with-holmium"
    args = ['--holmium-environment=staging'
            , '--holmium-browser=chrome'
            , '--holmium-useragent="test useragent"'
            , '--holmium-remote=http://nowhere.org'
            , "--holmium-capabilities=%s" % json.dumps({"foo":1})
            , "--holmium-ignore-ssl-errors"
            , "--holmium-browser-per-test"
            ]
    suitepath = os.path.join(support, 'options')
    plugins = [holmium.core.HolmiumNose()]
    def setUp(self):
        self.old_mapping = holmium.core.noseplugin.BROWSER_MAPPING
        holmium.core.noseplugin.BROWSER_MAPPING = build_mock_mapping("remote")
        holmium.core.noseplugin.BROWSER_MAPPING.update(build_mock_mapping("chrome"))
        ENV.clear()
        super(TestOptionsWithChrome,self).setUp()
    def runTest(self):
        assert "ERROR" not in self.output, self.output
        assert "FAIL" not in self.output, self.output
    def tearDown(self):
        holmium.core.noseplugin.BROWSER_MAPPING = self.old_mapping

class TestOptionsWithPhantom(PluginTester, unittest.TestCase):
    activate = "--with-holmium"
    args = ['--holmium-environment=staging'
        , '--holmium-browser=phantomjs'
        , '--holmium-useragent="test useragent"'
        , '--holmium-remote=http://nowhere.org'
        , "--holmium-capabilities=%s" % json.dumps({"foo":1})
        , "--holmium-ignore-ssl-errors"
        , "--holmium-browser-per-test"
    ]
    suitepath = os.path.join(support, 'options')
    plugins = [holmium.core.HolmiumNose()]
    def setUp(self):
        self.old_mapping = holmium.core.noseplugin.BROWSER_MAPPING
        holmium.core.noseplugin.BROWSER_MAPPING = build_mock_mapping("remote")
        holmium.core.noseplugin.BROWSER_MAPPING.update(build_mock_mapping("phantomjs"))
        ENV.clear()
        super(TestOptionsWithPhantom,self).setUp()
    def runTest(self):
        assert "ERROR" not in self.output, self.output
        assert "FAIL" not in self.output, self.output
    def tearDown(self):
        holmium.core.noseplugin.BROWSER_MAPPING = self.old_mapping

class TestConfigPy(PluginTester, unittest.TestCase):
    activate = "--with-holmium"
    args = ['--holmium-environment=tenv'
            , '--holmium-browser=chrome'
            ]
    suitepath = os.path.join(support, 'config_py')
    plugins = [holmium.core.HolmiumNose()]
    def setUp(self):
        self.old_mapping = holmium.core.noseplugin.BROWSER_MAPPING
        holmium.core.noseplugin.BROWSER_MAPPING.update(build_mock_mapping("chrome"))
        ENV.clear()
        super(TestConfigPy,self).setUp()
    def runTest(self):
        assert "ERROR" not in self.output, self.output
        assert "FAIL" not in self.output, self.output
    def tearDown(self):
        holmium.core.noseplugin.BROWSER_MAPPING = self.old_mapping

class TestConfigJson(PluginTester, unittest.TestCase):
    activate = "--with-holmium"
    args = ['--holmium-environment=tenv'
            , '--holmium-browser=chrome'
            ]
    suitepath = os.path.join(support, 'config_json')
    plugins = [holmium.core.HolmiumNose()]
    def setUp(self):
        self.old_mapping = holmium.core.noseplugin.BROWSER_MAPPING
        holmium.core.noseplugin.BROWSER_MAPPING.update(build_mock_mapping("chrome"))
        ENV.clear()
        super(TestConfigJson,self).setUp()
    def runTest(self):
        assert "ERROR" not in self.output, self.output
        assert "FAIL" not in self.output, self.output
    def tearDown(self):
        holmium.core.noseplugin.BROWSER_MAPPING = self.old_mapping

class TestConfigBad(PluginTester, unittest.TestCase):
    activate = "--with-holmium"
    args = ['--holmium-environment=tenv'
            , '--holmium-browser=chrome'
            ]
    suitepath = os.path.join(support, 'config_bad')
    plugins = [holmium.core.HolmiumNose()]
    def setUp(self):
        self.old_mapping = holmium.core.noseplugin.BROWSER_MAPPING
        holmium.core.noseplugin.BROWSER_MAPPING.update(build_mock_mapping("chrome"))
        ENV.clear()
        super(TestConfigBad,self).setUp()
    def runTest(self):
        assert "ERROR: test_config_bad" in self.output, self.output
    def tearDown(self):
        holmium.core.noseplugin.BROWSER_MAPPING = self.old_mapping

class TestMultipleDriversWithBrowserPerTest(PluginTester, unittest.TestCase):
    activate = "--with-holmium"
    args = ['--holmium-environment=tenv'
        , '--holmium-browser=chrome'
        , '--holmium-browser-per-test'
    ]
    suitepath = os.path.join(support, 'multiple_drivers')
    plugins = [holmium.core.HolmiumNose(), Skip()]
    def setUp(self):
        self.old_mapping = holmium.core.noseplugin.BROWSER_MAPPING
        mock_browsers = build_mock_mapping("chrome")
        holmium.core.noseplugin.BROWSER_MAPPING.update(mock_browsers)
        ENV.clear()
        super(TestMultipleDriversWithBrowserPerTest,self).setUp()
    def runTest(self):
        assert "Ran 2 tests" in self.output, self.output
        assert "OK" in self.output, self.output
        self.assertAlmostEqual(holmium.core.noseplugin.BROWSER_MAPPING["chrome"].return_value.quit.call_count, 3)
        self.assertAlmostEqual(holmium.core.noseplugin.BROWSER_MAPPING["chrome"].call_count, 3)

    def tearDown(self):
        holmium.core.noseplugin.BROWSER_MAPPING = self.old_mapping

class TestMultipleDriversWithReusedBrowsers(PluginTester, unittest.TestCase):
    activate = "--with-holmium"
    args = ['--holmium-environment=tenv'
        , '--holmium-browser=chrome'
    ]
    suitepath = os.path.join(support, 'multiple_drivers')
    plugins = [holmium.core.HolmiumNose(), Skip()]
    def setUp(self):
        self.old_mapping = holmium.core.noseplugin.BROWSER_MAPPING
        mock_browsers = build_mock_mapping("chrome")
        holmium.core.noseplugin.BROWSER_MAPPING.update(mock_browsers)
        ENV.clear()
        super(TestMultipleDriversWithReusedBrowsers,self).setUp()
    def runTest(self):
        assert "Ran 2 tests" in self.output, self.output
        assert "OK" in self.output, self.output
        self.assertAlmostEqual(holmium.core.noseplugin.BROWSER_MAPPING["chrome"].call_count, 2)

    def tearDown(self):
        holmium.core.noseplugin.BROWSER_MAPPING = self.old_mapping

class TestDriverBroken(PluginTester, unittest.TestCase):
    activate = "--with-holmium"
    args = ['--holmium-environment=tenv'
        , '--holmium-browser=chrome'
    ]
    suitepath = os.path.join(support, 'broken_driver')
    plugins = [holmium.core.HolmiumNose(), Skip()]
    def setUp(self):
        self.old_mapping = holmium.core.noseplugin.BROWSER_MAPPING
        mock_browsers = build_mock_mapping("chrome")
        def fake_construct(*a, **k):
            raise Exception("failed to initialize")
        mock_browsers["chrome"].side_effect = fake_construct
        holmium.core.noseplugin.BROWSER_MAPPING.update(mock_browsers)
        ENV.clear()
        super(TestDriverBroken,self).setUp()
    def runTest(self):
        assert "Ran 1 test" in self.output, self.output
        assert "SKIP=1" in self.output, self.output
    def tearDown(self):
        holmium.core.noseplugin.BROWSER_MAPPING = self.old_mapping

class TestNoDriver(PluginTester, unittest.TestCase):
    activate = "--with-holmium"
    suitepath = os.path.join(support, 'broken_driver')
    plugins = [holmium.core.HolmiumNose(), Skip()]
    def setUp(self):
        self.old_mapping = holmium.core.noseplugin.BROWSER_MAPPING
        mock_browsers = build_mock_mapping("chrome")
        holmium.core.noseplugin.BROWSER_MAPPING.update(mock_browsers)
        ENV.clear()
        super(TestNoDriver,self).setUp()
    def runTest(self):
        assert "Ran 1 test" in self.output, self.output
        assert "SKIP=1" in self.output, self.output
    def tearDown(self):
        holmium.core.noseplugin.BROWSER_MAPPING = self.old_mapping

class TestCapabilitiesFromFile(PluginTester, unittest.TestCase):
    activate = "--with-holmium"
    args = [ '--holmium-browser=phantomjs'
        , "--holmium-capabilities=./tests/support/captest/caps.json"
    ]
    suitepath = os.path.join(support, 'captest')
    plugins = [holmium.core.HolmiumNose()]
    def setUp(self):
        ENV.clear()
        super(TestCapabilitiesFromFile,self).setUp()
    def runTest(self):
        assert "Ran 1 test" in self.output, self.output



import unittest
import selenium.webdriver
from holmium.core import Page, Element, Elements, Locators, ElementMap


class WikiPedia(Page):
    languages = ElementMap( Locators.CLASS_NAME, "central-featured-lang"
                                    , key = lambda el:el.get_attribute("lang")
                                    , value = lambda el: el.find_element_by_tag_name("a"))
    search_box = Element( Locators.CSS_SELECTOR, "input#searchInput" )
    article_title = Element( Locators.CSS_SELECTOR, "h1#firstHeading span[dir=auto]" )
    search_results = ElementMap( Locators.CSS_SELECTOR, "div.mw-search-result-heading>a")
    def search(self,  query ):
        self.search_box.clear()
        self.search_box.send_keys( query )
        self.search_box.submit()



class TextSearchArticle(unittest.TestCase):
    def setUp(self):
        self.driver = selenium.webdriver.Firefox()
        self.page = WikiPedia(self.driver, "http://wikipedia.org")

    def test_text_search_alllangs(self):
        for language in self.page.languages:
            self.page.go_home().languages[language].click()
            self.assertEquals(self.page.search("google").article_title.text, "Google", language)

    def tearDown(self):
        self.driver.quit()



"""
global logger
"""
import logging
#: shared stream logger for all holmium classes
# pylint: disable=invalid-name
log = logging.Logger("holmium.core")
log.setLevel(logging.INFO)
_shandler = logging.StreamHandler()
_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
formatter = logging.Formatter(_format)
_shandler.setFormatter(formatter)
log.addHandler(_shandler)



import unittest
import holmium.core

class TestOptions(unittest.TestCase):
    def test_browser_set(self):
        self.assertEqual(self.driver.name, "remote")
    def test_remote_set(self):
        self.config["remote"] = "{{holmium.remote}}"
        self.assertEqual(self.config["remote"], 'http://nowhere.org')

    def test_useragent_set(self):
        self.config["user_agent"] = "{{holmium.user_agent}}"
        self.assertEqual(self.config["user_agent"], '"test useragent"')




"""
tests for facets
"""
import unittest
from flask_testing import LiveServerTestCase
import mock

from holmium.core import Page, Element, Locators, Elements, Section, ElementMap
from holmium.core.facets import FacetError, defer, title, cookie, strict

from . import webapp
from tests.utils import get_driver


class LiveTest(LiveServerTestCase):
    def create_app(self):
        return webapp.create_app()
    def setUp(self):
        super(LiveTest, self).setUp()
        webapp.db.create_db()

@strict()
class NavSection(Section):
    links = Elements(Locators.CSS_SELECTOR, "ul>li")

@strict()
class BadNavSectionStrict(Section):
    links = Elements(Locators.CSS_SELECTOR, "ul>li")
    junk = Element(Locators.CLASS_NAME, "junk")

@strict(debug=True)
class BadNavSectionStrictDebug(Section):
    links = Elements(Locators.CSS_SELECTOR, "ul>li")
    junk = Element(Locators.CLASS_NAME, "junk")

class BadNavSection(Section):
    links = Elements(Locators.CSS_SELECTOR, "ul>bi", facet=True)

class JumboTron(Section):
    callout = Element(Locators.TAG_NAME, "h1")
    link = Element(Locators.CLASS_NAME, "btn")

class BasePage(Page):
    nav = NavSection(Locators.CLASS_NAME, "nav")
    jumbo = JumboTron(Locators.CLASS_NAME, "jumbotron")


@title(title="login")
class LoginPage(BasePage):
    email = Element(Locators.NAME, "email", facet=True)
    password = Element(Locators.NAME, "password", facet=True)
    submit = Element(Locators.ID, "login-submit", facet=True)

    def login(self, email, password):
        self.email.send_keys(email)
        self.password.send_keys(password)
        self.submit.click()


@title(title="main page")
@cookie(name="uid")
@defer(page=LoginPage,
                    action=LoginPage.login,
                    action_arguments={"email": "john@doe.com",
                                      "password": "sekret"},
                    required=False)
class MainPage(BasePage):
    references = ElementMap ( Locators.CLASS_NAME, "reference-link")
    reference_content = Element(Locators.ID, "reference-content")
    selenium = Element(Locators.LINK_TEXT, "Selenium", timeout=5)


def do_login(page, email, password):
   page.login(email, password)

@title(title="main page")
@cookie(name="uid")
@defer(page=LoginPage,
                    action=do_login,
                    action_arguments={"email": "john@doe.com",
                                      "password": "sekret"},
                    required=True)
class MainPageWithCallable(BasePage):
    references = ElementMap ( Locators.CLASS_NAME, "reference-link")
    reference_content = Element(Locators.ID, "reference-content")
    selenium = Element(Locators.LINK_TEXT, "Selenium", timeout=5)

class MainPageBadElement(BasePage):
    bad_element = Element(Locators.NAME, "bad_element", facet=True)

class MainPageBadSectionElement(BasePage):
    nav = BadNavSection(Locators.CLASS_NAME, "nav")

class MainPageMixedSection(BasePage):
    bad_nav = BadNavSection(Locators.CLASS_NAME, "nav")


class FacetsTests(LiveTest):
    def setUp(self):
        super(FacetsTests, self).setUp()
        self.driver = get_driver()
        self.base_url = self.get_server_url()

    def test_main_page_callout(self):
        main = MainPage(self.driver, self.base_url)
        self.assertEqual(main.jumbo.callout.text, "Here's a few ways it can be good")

    def test_main_page_with_callable_callout(self):
        main = MainPageWithCallable(self.driver, self.base_url)
        self.assertEqual(main.jumbo.callout.text, "Here's a few ways it can be good")

    def test_main_click_selenium(self):
        main = MainPage(self.driver, self.base_url)
        main.selenium.click()
        self.assertTrue("Selenium automates browsers." in main.reference_content.text)

    def test_main_page_login_first(self):
        login = LoginPage(self.driver, self.base_url + "/login")
        login.login("john@doe.com", "sekret")
        main = MainPage(self.driver, self.base_url)
        self.assertEqual(main.jumbo.callout.text, "Here's a few ways it can be good")

    def test_main_page_bad_trait_element(self):
        main = MainPageBadElement(self.driver, self.base_url)
        self.assertRaises(FacetError, lambda:main.jumbo)

    def test_main_page_bad_section_element(self):
        main = MainPageBadSectionElement(self.driver, self.base_url)
        self.assertRaises(FacetError, lambda: main.nav.links)

    def test_main_page_bad_section_strict(self):
        class _P(MainPage):
            bad_nav = BadNavSectionStrict(Locators.CLASS_NAME, "nav")
        main = _P(self.driver, self.base_url)
        self.assertRaises(FacetError, lambda: main.bad_nav.links)
        self.assertTrue( main.nav.links is not None)

    def test_main_page_bad_section_strict_debug(self):
        class _P(MainPage):
            bad_nav = BadNavSectionStrictDebug(Locators.CLASS_NAME, "nav")
        main = _P(self.driver, self.base_url)
        with mock.patch("holmium.core.facets.log") as log:
            self.assertTrue( main.bad_nav.links is not None)
            self.assertTrue( main.bad_nav.junk is None)
            self.assertTrue( main.nav.links is not None)
            self.assertTrue(log.warn.call_count == 1)
            self.assertTrue("failed to exhibit facet junk" in str(log.warn.call_args))

    def test_main_page_good_and_bad_section_element(self):
        main = MainPageMixedSection(self.driver, self.base_url)
        self.assertRaises(FacetError, lambda: main.bad_nav.links)
        self.assertTrue( main.nav.links is not None)

    def tearDown(self):
        super(FacetsTests, self).tearDown()

class GoodFacetsTest(unittest.TestCase):
    def test_all(self):

        class Other(Page):
            def do(self):
                return

        @title(title="title")
        @cookie(name="cookie", value="yum")
        @cookie(name="cookie", value=lambda c:c=="yum")
        @defer(page=Other, action=Other.do)
        class P(Page):
            el = Element(Locators.CLASS_NAME, "null")

        driver = mock.Mock()
        driver.find_element.return_value.text = "null"
        driver.get_cookie.return_value = "yum"
        driver.title = "title"
        p = P(driver)
        self.assertEqual(p.el.text, "null")

class BadFacetTests(unittest.TestCase):
    def test_missing_title(self):
        @title(title="foo")
        class P(Page):
            el = Element(Locators.CLASS_NAME, "null")

        driver = mock.Mock()
        p = P(driver)
        self.assertRaises(FacetError, lambda: p.el)

    def test_failed_callable(self):
        @cookie(name="foo", value=lambda c:c==1)
        class P(Page):
            el = Element(Locators.CLASS_NAME, "null")

        driver = mock.Mock()
        driver.get_cookie.return_value = 2
        p = P(driver)
        self.assertRaises(FacetError, lambda: p.el)

class BadReactionTests(unittest.TestCase):
    def test_failed_reaction_required(self):

        class Other(Page):
            def do(self):
                raise Exception("can't")
        @defer(page=Other, action=Other.do)
        class P(Page):
            el = Element(Locators.CLASS_NAME, "null")

        driver = mock.Mock()
        p = P(driver)
        self.assertRaises(FacetError, lambda: p.el)
    def test_passed_reaction_notrequired(self):

        class Other(Page):
            def do(self):
                raise Exception("can't")
        @defer(page=Other, action=Other.do, required=False)
        class P(Page):
            el = Element(Locators.CLASS_NAME, "null")

        driver = mock.Mock()
        driver.find_element.return_value.text = "null"
        p = P(driver)
        self.assertEqual(p.el.text, "null")


def test_missing_arguments():
    for args in [{}, {"page":1}, {"page":1, "action":1, "foobar":1}]:
        yield check_arguments, args

def check_arguments(args):
    def create(**kw):
        @defer(**kw)
        class P(Page):
            pass
    try:
        create(**args)
        raise Exception("exception not raised")
    except AttributeError as e:
        pass




# -*- coding: utf-8 -*-
import unittest
import threading
from holmium.core.facets import cookie,title, defer
from nose.plugins.attrib import attr
import os
from selenium.webdriver.support.ui import Select
import mock

from holmium.core import Element, ElementEnhancer, Page, Locators
from holmium.core import reset_enhancers, register_enhancer

support = os.path.join(os.path.dirname(__file__), "support")


class BugReports(unittest.TestCase):
    def test_multiple_pageinstances(self):
        """ https://github.com/alisaifee/holmium.core/issues/4
        """
        class p(Page):
            el = Element(Locators.NAME, "name")
        d1,d2=mock.Mock(), mock.Mock()
        p1,p2=p(d1, "http://p1"),p(d2, "http://p2")
        e1,e2=mock.Mock(), mock.Mock()
        e2.tag_name = e1.tag_name = "div"
        e1.text = "t1"
        e2.text = "t2"
        d1.find_element.return_value = e1
        d2.find_element.return_value = e2
        self.assertEqual(p1.el.text,"t1")
        self.assertEqual(p2.el.text,"t2")
        self.assertEqual(d1.get.call_count, 1)
        self.assertEqual(d2.get.call_count, 1)
    def test_multiple_pageinstances_multithreaded(self):
        """ https://github.com/alisaifee/holmium.core/issues/4
        """
        class p(Page):
            el = Element(Locators.NAME, "name")
        class p2(Page):
            el = Element(Locators.NAME, "name")

        def exec_page_in_thread(p):
            p.go_home()
            self.assertEqual(p.t, p.el.text)
            self.assertEqual(p.driver.get.call_count, 2)

        def build_pages(po):
            pages=[]
            for i in range(0,100):
                d = mock.Mock()
                e = mock.Mock()
                e.text = str(p)+str(i)
                e.tag = "div"
                d.find_element.return_value = e
                _p = po(d,"http://%s" % i)
                _p.t = str(p)+str(i)
                pages.append(_p)
            return pages
        pages = build_pages(p)
        pages.extend(build_pages(p2))

        threads = [threading.Thread(target=exec_page_in_thread, args=(p,)) for p in pages]
        [k.start() for k in threads]
        [k.join() for k in threads]

    def test_fluent_response(self):
        """ https://github.com/alisaifee/holmium.core/issues/8
        """
        class p(Page):
            el = Element(Locators.NAME, "name")
            def f1(self):
                self.el.click()
            def f2(self):
                return False
            def f3(self):
                return []
            def f4(self):
                return {}
            def f5(self):
                return ""
        d1=mock.Mock()
        p1=p(d1, "http://p1")
        self.assertEqual(p1, p1.f1())
        self.assertEqual(False, p1.f2())
        self.assertEqual([], p1.f3())
        self.assertEqual({}, p1.f4())
        self.assertEqual("",p1.f5())

    def test_select_element(self):
        """ https://github.com/alisaifee/holmium.core/issues/13
        """
        class SimplePage(Page):
            id_el = Element(Locators.ID, "simple_id")
        driver = mock.Mock()
        driver.find_element.return_value.tag_name = "select"
        self.assertTrue(isinstance(SimplePage(driver).id_el, Select))
        driver.find_element.return_value.tag_name = "SELECT"
        self.assertTrue(isinstance(SimplePage(driver).id_el, Select))

    def test_custom_enhancement(self):
        """ https://github.com/alisaifee/holmium.core/issues/14
        """
        class CustomSelect(ElementEnhancer):
            __TAG__ = "select"
            def get_text_upper(self):
                return self.element.text.upper()

        register_enhancer(CustomSelect)
        class SimplePage(Page):
            id_el = Element(Locators.ID, "simple_id")
        driver = mock.Mock()
        driver.find_element.return_value.tag_name = "select"
        driver.find_element.return_value.text = "fOo"
        self.assertTrue(issubclass(SimplePage(driver).id_el.__class__, CustomSelect))
        self.assertEquals(SimplePage(driver).id_el.get_text_upper(), "FOO")

    @attr("sigh")
    def test_class_inheritance_with_facets(self):
        """ https://github.com/alisaifee/issues/18
        """
        @cookie(name="foo")
        class B1(Page):
            def dance(self):
                return

        @title(title="one")
        class ExtOne(B1):
            pass

        @defer(page=ExtOne, action=ExtOne.dance)
        class ExtTwo(B1):
            pass

        @title(title="three")
        @cookie(name="bar")
        class ExtThree(ExtOne, ExtTwo):
            pass

        @title(title="four")
        class B2(Page):
            pass

        @cookie(name="foo")
        class B3(Page):
            pass

        class ExtFour(B2,B3):
            pass


        self.assertEquals(len(B1.get_class_facets()), 1)
        self.assertEquals(len(ExtOne.get_class_facets()), 2)
        self.assertEquals(len(ExtTwo.get_class_facets()), 2)
        self.assertEquals(len(ExtThree.get_class_facets()), 4)
        # ensure the last title is the one used
        self.assertEquals(ExtThree.get_class_facets().type_map[title].pop().arguments,
            {"title":"three"})

        self.assertEquals(len(ExtFour.get_class_facets()), 2)

    def tearDown(self):
        reset_enhancers()
        super(BugReports,self).tearDown()



import unittest
import hiro
import time
from tests.utils import make_temp_page, get_driver
from holmium.core import Page, Element, Locators
from holmium.core.conditions import *

class WaitConditionTests(unittest.TestCase):
    def setUp(self):
        self.driver = get_driver()
        self.uri = make_temp_page("<div id='simple_id'>default text</div>")

    def run_script(self, script, delay=0, sync=False):
        return (hiro.run_sync if sync else hiro.run_async)(1, lambda: time.sleep(delay) or self.driver.execute_script(script))

    def build_page_object(self, condition):
        class P(Page):
            id_el = Element(Locators.ID, "simple_id"
                , only_if=condition
                , timeout=5)
        return P

    @hiro.Timeline(scale=10)
    def test_only_if_matches_text(self):
        page = self.build_page_object(MATCHES_TEXT("changed"))(self.driver, self.uri)
        self.assertTrue(page.id_el is None)
        runner = self.run_script('document.getElementById("simple_id").innerHTML="changed";', 1)
        self.assertEqual(page.id_el.text, "changed")


    @hiro.Timeline(scale=10)
    def test_only_if_displayed(self):
        page = self.build_page_object(VISIBLE())(self.driver, self.uri)
        runner = self.run_script('document.getElementById("simple_id").style.display="none";', sync=True)
        runner = self.run_script('document.getElementById("simple_id").style.display="block";', 1)
        self.assertTrue(page.id_el is not None)

    @hiro.Timeline(scale=10)
    def test_only_if_invisible(self):
        page = self.build_page_object(INVISIBLE())(self.driver, self.uri)
        self.assertTrue(page.id_el is None)
        runner = self.run_script('document.getElementById("simple_id").style.display="none";', 1)
        self.assertTrue(page.id_el is not None)




"""
aliases so that a real error message is displayed if someone
uses the old class named
"""
from .pageobject import Page, Element, Elements, ElementMap
from .testcase import TestCase

# pylint: disable=invalid-name
# pylint: disable=too-few-public-methods
# pylint: disable=no-member

class Deprecated(object):
    """
    meta class to create an object that throws a Syntax error on
    construction
    """
    def __new__(cls, *_):
        raise SyntaxError(
            "%s has been removed as of version 0.4. Use %s instead" % (
                cls.cur, cls.alt.__name__
            )
        )


def construct_deprecated(name, alt):
    """
    create a type for the alias
    """
    doc = """Deprecated alias for :class:`%s`""" % alt.__name__
    cls = type(name, (Deprecated, alt),
               dict(cur=name, alt=alt, __doc__=doc))
    return cls


PageObject = construct_deprecated("PageObject", Page)
PageElement = construct_deprecated("PageElement", Element)
PageElements = construct_deprecated("PageElements", Elements)
PageElementMap = construct_deprecated("PageElementMap", ElementMap)
HolmiumTestCase = construct_deprecated("HolmiumTestCase", TestCase)




import unittest
import time
from  holmium.core import Page, Element, Locators
from tests.utils import get_driver, make_temp_page
import hiro

class ElementTest(unittest.TestCase):
    page_content = """
            <body>
                <div id="simple_id">simple_id</div>
                <div class="simple_class">simple_class</div>
                <div class="simple_xpath"><h3>Simple XPATH</h3></div>
            </body>
        """

    def setUp(self):
        self.driver = get_driver()

    def test_basic_element(self):
        class SimplePage(Page):
            id_el = Element(Locators.ID, "simple_id")
            class_el = Element(Locators.CLASS_NAME, "simple_class")
            selector_el = Element(Locators.CSS_SELECTOR, "div.simple_class")
            xpath_el = Element(Locators.XPATH, "//div[h3/text()='Simple XPATH']")


        uri = make_temp_page(ElementTest.page_content)
        page = SimplePage(self.driver, uri)
        assert page.id_el.text == "simple_id"
        assert page.class_el.text == "simple_class"
        assert page.selector_el.text == "simple_class"
        assert page.xpath_el.text == "Simple XPATH"


    def test_basic_element_with_dict(self):
        class SimplePage(Page):
            elements = { "id": Element ( Locators.ID, "simple_id" ), "class" : Element(Locators.CLASS_NAME, "simple_class") }


        uri = make_temp_page(ElementTest.page_content)
        page = SimplePage(self.driver, uri)
        assert page.elements["id"].text == "simple_id"
        assert page.elements["class"].text == "simple_class"

    def test_basic_element_with_list(self):
        class SimplePage(Page):
            elements = [Element ( Locators.ID, "simple_id" ), Element(Locators.CLASS_NAME, "simple_class") ]

        uri = make_temp_page(ElementTest.page_content)
        page = SimplePage(self.driver, uri)
        assert page.elements[0].text == "simple_id"
        assert page.elements[1].text == "simple_class"


    def test_basic_element_with_only_if(self):

        class SimplePage(Page):
            id_el = Element(Locators.ID, "simple_id")
            id_el_changed = Element(Locators.ID, "simple_id", timeout=10,
                                    only_if=lambda el: el.text == "changed")

        uri = make_temp_page(ElementTest.page_content)
        page = SimplePage(self.driver, uri)
        self.assertEqual(page.id_el.text, "simple_id")
        script = 'document.getElementById("simple_id").firstChild.nodeValue="changed";'
        runner = hiro.run_async(5, lambda: time.sleep(1) or self.driver.execute_script(script))
        with hiro.Timeline().scale(10):
            self.assertEqual(page.id_el_changed.text, "changed")
        self.assertTrue(runner.get_response() == None)
        self.driver.refresh()
        with hiro.Timeline().scale(10):
            self.assertEqual(page.id_el_changed, None)



"""
setup.py for holmium.core


"""
__author__ = "Ali-Akber Saifee"
__email__ = "ali@indydevs.org"
__copyright__ = "Copyright 2014, Ali-Akber Saifee"

from setuptools import setup, find_packages
import os
import sys
this_dir = os.path.abspath(os.path.dirname(__file__))
REQUIREMENTS = filter(None, open(os.path.join(this_dir, 'requirements.txt')).read().splitlines())

import versioneer
versioneer.versionfile_source = "holmium/core/_version.py"
versioneer.versionfile_build = "holmium/core/version.py"
versioneer.tag_prefix = ""
versioneer.parentdir_prefix = "holmium.core-"

setup(
    name='holmium.core',
    author = __author__,
    author_email = __email__,
    license = open("LICENSE.txt").read(),
    url="https://holmiumcore.readthedocs.org/en/latest/",
    zip_safe = False,
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    install_requires = REQUIREMENTS,
    classifiers=[k for k in open('CLASSIFIERS').read().split('\n') if k],
    description='selenium page objects and other utilities for test creation',
    long_description=open('README.rst').read() + open('HISTORY.rst').read(),
    packages = find_packages(exclude=["tests*"]),
    entry_points = {
        'nose.plugins.0.10': ['holmium = holmium.core:HolmiumNose',]
        }
)




"""
implementation of element enhancer base
"""


from selenium.webdriver.support.ui import Select


class ElementEnhancer(object):
    """
    base class for implementing custom element enhancers to
    add functionality to located webelements based on the
    element type (tag name)
    """
    # pylint: disable=too-few-public-methods
    __TAG__ = ""
    def __init__(self, element):
        self.element = element
        super(ElementEnhancer, self).__init__()

    @classmethod
    def matches(cls, element):
        """
        class method to verify that this enhancer is appropriate
        for the provided webelement
        """
        return cls.__TAG__.lower() == element.tag_name.lower()

    def __getattribute__(self, key):
        element = object.__getattribute__(self, "element")
        try:
            try:
                return super(ElementEnhancer, self).__getattribute__(key)
            except AttributeError:
                return element.__getattribute__(key)
        except:
            raise AttributeError(
                "neither %s, nor %s object has an attribute %s" % (
                self.__class__.__name__, element.__class__.__name__, key))


class _SelectMixin(Select, object):
    """
    cooperative super version of Select
    """
    def __init__(self):
        super(_SelectMixin, self).__init__(self.element)
        Select.__init__(self, self.element)

class SelectEnhancer(ElementEnhancer, _SelectMixin):
    """
    Enhancer for the select tag
    """
    __TAG__ = "select"


REGISTERED_ENHANCERS = [SelectEnhancer]

def register_enhancer(enhancer):
    """
    registers a :class:`ElementEnhancer` with the internal
    lookup
    """
    if not issubclass(enhancer, ElementEnhancer):
        raise TypeError(
            "Only subclasses of holmium.core.ElementEnhancer can be registered")
    if not hasattr(enhancer, "__TAG__") or not enhancer.__TAG__:
        raise AttributeError(
            "ElementEnhancer implementations must declare a __TAG__"
            "property to match against"
        )

    REGISTERED_ENHANCERS.insert(0, enhancer)

def reset_enhancers():
    """
    resets the state so that any :class:`ElementEnhancer` that was registered
    via a call to :func:`register_enhancer` is removed.
    """
    global REGISTERED_ENHANCERS
    REGISTERED_ENHANCERS = [SelectEnhancer]

def get_enhancers():
    """
    returns the global registered enhancers
    """
    return REGISTERED_ENHANCERS





import unittest
import os

import mock
from nose.plugins.attrib import attr
from nose.plugins.skip import SkipTest

from holmium.core import TestCase, Element, Elements, ElementMap, Locators
from holmium.core.env import ENV
import holmium.core.testcase

from .utils import mock_open, build_mock_mapping


def runtc(env, validations):
    try:
        _pre_env = dict(os.environ)
        os.environ.update(env)
        class t1(TestCase):
            def runTest(self):
                self.driver.get("http://nowhere.org")
                for validation in validations:
                    self.assertTrue(validation(self))
        test = t1()
        test.setUpClass()
        test.setUp()
        test.runTest()
        test.tearDown()
        test.tearDownClass()
    finally:
        os.environ = _pre_env

def runtc2drivers(env, validations):
    try:
        _pre_env = dict(os.environ)
        os.environ.update(env)
        class t2(TestCase):
            def runTest(self):
                self.driver.get("http://nowhere.org")
                self.drivers[0].get("http://nowhere.org")
                self.drivers[1].get("http://nowhere.org")
                for validation in validations:
                    self.assertTrue(validation(self))
        test = t2()
        test.setUpClass()
        test.setUp()
        test.runTest()
        test.tearDown()
        test.tearDownClass()
    finally:
        os.environ = _pre_env

class TestCaseTests(unittest.TestCase):
    def setUp(self):
        if "driver" in ENV:
            ENV.pop("driver")

    def test_set_browser(self):
        with mock.patch.dict('holmium.core.testcase.BROWSER_MAPPING', build_mock_mapping("phantomjs")):
            runtc({"HO_BROWSER":"phantomjs"}, [lambda s:s.driver!=None])

    def test_set_browser_phantom(self):
        with mock.patch.dict('holmium.core.testcase.BROWSER_MAPPING', build_mock_mapping("phantomjs")):
            runtc({"HO_BROWSER":"phantomjs"}, [lambda s:s.driver!=None, lambda s:s.driver.name == "phantomjs"])

    def test_set_browser_remote(self):
        with mock.patch.dict('holmium.core.testcase.BROWSER_MAPPING', build_mock_mapping("remote")):
            runtc({"HO_BROWSER":"phantomjs", "HO_REMOTE":"http://lala.com"}, [lambda s:s.driver!=None, lambda s:s.driver.name == "remote"])

    def test_auto_config_json(self):
        with mock.patch("os.path.isfile") as isfile:
            with mock.patch.dict('holmium.core.testcase.BROWSER_MAPPING', build_mock_mapping("phantomjs")):
                isfile.side_effect = lambda name:name.find("json")>0
                with mock_open() as op:
                    op.return_value.read.return_value = '{"default": {"test": 1}}'
                    runtc({"HO_BROWSER":"phantomjs"}, [lambda s:s.config["test"]==1])

    def test_auto_config_py(self):
        with mock.patch("os.path.isfile") as isfile:
            with mock.patch.dict('holmium.core.testcase.BROWSER_MAPPING', build_mock_mapping("phantomjs")):
                isfile.side_effect = lambda name:name.find("py")>0
                with mock.patch("imp.load_source") as load_source:
                    m_config = mock.Mock()
                    m_config.config = {"default":{"test":1}}
                    load_source.return_value = m_config
                    runtc({"HO_BROWSER":"phantomjs"}, [lambda s:s.config["test"]==1])

    def test_set_useragent(self):
        with mock.patch.dict('holmium.core.testcase.BROWSER_MAPPING', build_mock_mapping("firefox")):
            runtc({"HO_BROWSER":"firefox", "HO_USERAGENT":"holmium.core"}, [])
            call_args = holmium.core.testcase.BROWSER_MAPPING["firefox"].call_args
            assert len(call_args) == 2
            assert "firefox_profile" in call_args[1]
            ff_profile = call_args[1]["firefox_profile"]
            ua = ff_profile.default_preferences["general.useragent.override"]
            self.assertEqual(ua, 'holmium.core')

    def test_invalid_browser(self):
        self.assertRaises(SkipTest, runtc, {"HO_BROWSER":"direfox", "HO_BROWSER_PER_TEST":"1"}, [])

    def test_browser_per_test(self):
        with mock.patch.dict('holmium.core.testcase.BROWSER_MAPPING', build_mock_mapping("firefox")):
            runtc({"HO_BROWSER":"firefox", "HO_BROWSER_PER_TEST":"1"}, [])
            self.assertEquals(holmium.core.testcase.BROWSER_MAPPING["firefox"].return_value.quit.call_count, 1)
        ENV.clear()
        with mock.patch.dict('holmium.core.testcase.BROWSER_MAPPING', build_mock_mapping("firefox")):
            runtc({"HO_BROWSER":"firefox", "HO_BROWSER_PER_TEST":"0"}, [])
            self.assertEquals(holmium.core.testcase.BROWSER_MAPPING["firefox"].return_value.quit.call_count, 0)

    def test_multiple_browser_per_test(self):
        with mock.patch.dict('holmium.core.testcase.BROWSER_MAPPING', build_mock_mapping("firefox")):
            runtc2drivers({"HO_BROWSER":"firefox", "HO_BROWSER_PER_TEST":"0"}, [])
            self.assertEquals(holmium.core.testcase.BROWSER_MAPPING["firefox"].call_count, 2)
        ENV.clear()
        with mock.patch.dict('holmium.core.testcase.BROWSER_MAPPING', build_mock_mapping("firefox")):
            runtc2drivers({"HO_BROWSER":"firefox", "HO_BROWSER_PER_TEST":"1"}, [])
            self.assertEquals(holmium.core.testcase.BROWSER_MAPPING["firefox"].call_count, 2)
            self.assertEquals(holmium.core.testcase.BROWSER_MAPPING["firefox"].return_value.quit.call_count, 2)




# -*- coding: utf-8 -*-
#

import sys, os
sys.path.insert(0, os.path.abspath('../../'))
import holmium.core

extensions = ['sphinx.ext.autodoc', 'sphinx.ext.autosummary','sphinx.ext.doctest', 'sphinx.ext.intersphinx', 'sphinx.ext.todo', 'sphinx.ext.pngmath', 'sphinx.ext.ifconfig', 'sphinx.ext.viewcode', 'sphinxcontrib.email']
templates_path = ['_templates']
source_suffix = '.rst'
master_doc = 'index'
project = u'holmium.core'
copyright = u'2014, Ali-Akber Saifee'
version = release = holmium.core.__version__

exclude_patterns = []
#add_module_names = True
pygments_style = 'sphinx'
#html_theme_options = {}
#html_theme_path = []
html_title = "holmium"
#html_favicon = None
html_static_path = ['_static']
htmlhelp_basename = 'holmiumcoredoc'

latex_documents = [
  ('index', 'holmiumcore.tex', u'holmium.core Documentation',
   u'Ali-Akber Saifee', 'manual'),
]
man_pages = [
    ('index', 'holmiumcore', u'holmium.core Documentation',
     [u'Ali-Akber Saifee'], 1)
]

texinfo_documents = [
  ('index', 'holmiumcore', u'holmium.core Documentation',
   u'Ali-Akber Saifee', 'holmiumcore', 'One line description of project.',
   'Miscellaneous'),
]
intersphinx_mapping = {'http://docs.python.org/': None, "http://selenium-python.readthedocs.org/en/latest":None, "http://jinja.pocoo.org/docs/" : None}



import unittest
import hiro
from holmium.core import ElementMap, Locators, Page
from tests.utils import get_driver, make_temp_page


class ElementMapTest(unittest.TestCase):
    def setUp(self):
        self.driver = get_driver()

    def test_basic_element(self):
        class SimplePage(Page):
            el_map_default = ElementMap(Locators.CLASS_NAME, "simple_class")
            el_map_keymapper = ElementMap(Locators.CLASS_NAME, "simple_class"
                                                            , key = lambda el: el.find_element_by_tag_name("a").text)
            el_map_valuemapper = ElementMap(Locators.CLASS_NAME, "simple_class"
                                                            , value = lambda el: el.find_element_by_tag_name("a").text)
            el_map_keyvaluemapper = ElementMap(Locators.CLASS_NAME, "simple_class"
                                                            , key = lambda el: el.find_element_by_tag_name("a").text
                                                            , value = lambda el: el.find_element_by_tag_name("a").get_attribute("href"))


        uri = make_temp_page("""
            <body>
                <div class="simple_class">
                    simple class el 1
                    <a href="http://el1.com/">element 1</a>
                </div>
                <div class="simple_class">
                    simple class el 2
                    <a href="http://el2.com/">element 2</a>
                </div>
                <div class="simple_class">
                    simple class el 3
                    <a href="http://el3.com/">element 3</a>
                </div>
            </body>
        """)
        page = SimplePage(self.driver, uri)
        self.assertEqual(list(page.el_map_default.keys()) , ["simple class el 1 element 1", "simple class el 2 element 2", "simple class el 3 element 3"] )
        self.assertEqual(list(page.el_map_keymapper.keys()) , ["element 1", "element 2", "element 3"] )
        self.assertEqual(list(page.el_map_valuemapper.values()) , ["element 1", "element 2", "element 3"] )
        self.assertEqual(list(page.el_map_keyvaluemapper.keys()) , ["element 1", "element 2", "element 3"] )
        self.assertEqual(list(page.el_map_keyvaluemapper.values()) , ["http://el1.com/", u"http://el2.com/", u"http://el3.com/"])

    def test_missing_elements(self):
        class SimplePage(Page):
            el_list = ElementMap(Locators.CLASS_NAME,
                                            "element")
            el_list_wait = ElementMap(
                Locators.CLASS_NAME, "elements", timeout=1)
            el_list_only_if = ElementMap(
                Locators.CLASS_NAME, "elements", timeout=1,
                only_if=lambda els: len(els) == 1)

        uri = make_temp_page("""
            <body>
                <div class="_">
                </div>
            </body>
        """)
        with hiro.Timeline().scale(10):
            page = SimplePage(self.driver, uri)
            self.assertEqual(page.el_list, {})
            self.assertEqual(page.el_list_wait, {})
            self.assertEqual(page.el_list_only_if, {})
            uri = make_temp_page("""
                <body>
                    <div class="elements">
                        element_text
                    </div>
                </body>
            """)
            page = SimplePage(self.driver, uri)
            self.assertEqual(page.el_list, {})
            self.assertEqual(len(page.el_list_wait), 1)
            self.assertEqual(len(page.el_list_only_if), 1)


"""
the testcase base class
"""
import unittest
import inspect
import imp
import json
import os
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support.wait import WebDriverWait
from .config import HolmiumConfig, Config, BROWSER_MAPPING
from .env import ENV
from .env import LazyWebDriver, LazyWebDriverList
from nose.plugins.skip import SkipTest

# pylint: disable=too-many-public-methods

class TestCase(unittest.TestCase):
    """
    Base class for creating test classes for writing holmium driven
    test cases. More details can be found at :ref:`testing-unittest`
    """

    @classmethod
    def setUpClass(cls):
        """
        prepare the driver initialization based on the environment variables
        that have been set. The driver is not actually initialized until the
        test itself actually refers to it via `self.driver`.
        """
        cls.driver = None
        base_file = inspect.getfile(cls)
        config_path = os.path.join(os.path.split(base_file)[0], "config")
        browser = os.environ.get("HO_BROWSER", "firefox")
        user_agent = os.environ.get("HO_USERAGENT", "")
        remote = os.environ.get("HO_REMOTE", None)
        environment = os.environ.get("HO_ENV", "development")
        ignore_ssl = os.environ.get("HO_IGNORE_SSL_ERRORS", False)
        fresh_instance = bool(int(os.environ.get("HO_BROWSER_PER_TEST", 0)))
        cls.holmium_config = HolmiumConfig(browser, remote, {}, user_agent,
                                       environment, ignore_ssl, fresh_instance)
        config = None
        if os.path.isfile(config_path + ".json"):
            config = json.loads(open(config_path + ".json").read())
        elif os.path.isfile(config_path + ".py"):
            config = imp.load_source("config", config_path + ".py").config
        if config:
            cls.config = Config(config, {"holmium": cls.holmium_config})
        # pylint: disable=no-member
        if remote:
            driver_cls = BROWSER_MAPPING["remote"]
        else:
            if not cls.holmium_config.browser in BROWSER_MAPPING:
                raise SkipTest("Unknown browser (%s) specified in HO_BROWSER"
                               % (cls.holmium_config.browser)
                )
            driver_cls = BROWSER_MAPPING[cls.holmium_config.browser]
        cls.driver = ENV.setdefault("driver", LazyWebDriver(driver_cls,
                                                            cls.holmium_config)
        )
        cls.drivers = ENV.setdefault("drivers", LazyWebDriverList())
        if hasattr(super(TestCase, cls), "setUpClass"):
            super(TestCase, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        """
        quit the driver after the test run (or after all the test methods
        in the class have finished if ``HO_BROWSER_PER_TEST`` is set).
        """
        # pylint:disable=no-member
        if ENV.get("driver", None) and cls.holmium_config.fresh_instance:
            for driver in ENV["drivers"]:
                driver.safe_quit()
        if hasattr(super(TestCase, cls), "tearDownClass"):
            super(TestCase, cls).tearDownClass()

    def tearDown(self):
        """
        clear the cookies on the driver after each test
        """
        if ENV.get("driver", None):
            for driver in ENV["drivers"]:
                driver.safe_clear()
        super(TestCase, self).tearDown()


    # helper assertions
    # pylint: disable=invalid-name
    def assertElementTextEqual(self, element, text, msg=None):
        """
        Fail if the text attribute of the element does not match
        """
        self.assertEqual(element.text, text, msg)

    def assertElementAttributeEqual(self, element, key, value, msg=None):
        """
        Fail if the element does not have the specified attribute value
        """
        self.assertEqual(element.get_attribute(key), value, msg)

    def assertElementDisplayed(self, element, msg=None):
        """
        Fail if the element is not visible
        """
        self.assertTrue(element.is_displayed(), msg)

    def assertElementsDisplayed(self, elements, msg=None):
        """
        Fail if any of the elements in the element collection are not
        visible
        """
        _ = elements.values() if isinstance(elements, dict) else elements
        self.assertTrue(all((el.is_displayed() for el in _)), msg)

    def assertElementCSS(self, element, css_property, value, msg=None):
        # pylint:disable=line-too-long
        """
        Fail if the element does not exhibit the correct css property value.
        The value of the elements css property is the one returned by
        :meth:`selenium.webdriver.remote.webelement.WebElement.value_of_css_property`
        """
        self.assertEqual(element.value_of_css_property(css_property),
                         value,
                         msg
        )

    def assertElementSize(self, element, width, height, msg=None):
        """ Fail if the element size does not match the provided values
        """
        _expected = {"height":height, "width":width}
        self.assertEqual(_expected, element.size, msg)


    def assertConditionWithWait(self, driver, condition, element,
                                timeout=0, msg=None):
        # pylint:disable=line-too-long
        """ Fail if the condition specified does not hold for the element within
        the specified timeout

        :param driver: the selenium driver
        :param condition: an instance of :mod:`selenium.webdriver.support.expected_conditions`
        :param element: the :class:`selenium.webdriver.remote.webelement.WebElement`
        """
        try:
            wait = WebDriverWait(driver, timeout)
            wait.until(condition(element))
        except TimeoutException:
            _msg = self._formatMessage(msg, "Timeout waiting on condition %s" % condition)
            self.failureException(_msg)
